@@ -1,6 +1,6 @@
 
 /*
- *  Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ *  Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to
@@ -25,7 +25,7 @@
 #include "E.h"
 
 void               *
-FindItem(char *name, int id, int find_by, int type)
+FindItem(const char *name, int id, int find_by, int type)
 {
    List               *ptr;
 
@@ -72,7 +72,7 @@ FindItem(char *name, int id, int find_by, int type)
 }
 
 void
-AddItem(void *item, char *name, int id, int type)
+AddItem(void *item, const char *name, int id, int type)
 {
    List               *ptr;
 
@@ -81,7 +81,7 @@ AddItem(void *item, char *name, int id, int type)
    if (!ptr)
       EDBUG_RETURN_;
    ptr->item = item;
-   ptr->name = duplicate(name);
+   ptr->name = Estrdup(name);
    ptr->id = id;
    ptr->type = type;
    ptr->next = lists.next;
@@ -260,7 +260,7 @@ ListItems(int *num, int type)
 	  {
 	     if (ptr->type == type)
 	       {
-		  list[i] = duplicate(ptr->name);
+		  list[i] = Estrdup(ptr->name);
 		  i++;
 	       }
 	     ptr = ptr->next;
@@ -270,7 +270,7 @@ ListItems(int *num, int type)
      {
 	while (ptr)
 	  {
-	     list[i] = duplicate(ptr->name);
+	     list[i] = Estrdup(ptr->name);
 	     i++;
 	     ptr = ptr->next;
 	  }
