@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -31,32 +31,35 @@ DockIt(EWin * ewin)
    char                id[32];
 
    EDBUG(3, "DockIt");
-   Esnprintf(id, sizeof(id), "%i", ewin->client.win);
+   Esnprintf(id, sizeof(id), "%i", (unsigned)ewin->client.win);
    ac = 0;
    ic = FindItem("DEFAULT_DOCK_BUTTON", 0, LIST_FINDBY_NAME, LIST_TYPE_ICLASS);
-   if (mode.dockstartx >= 0)
+   if (Conf.dock.startx >= 0)
      {
-	bt = CreateButton(id, ic, ac, NULL, NULL, 1, 0, 64, 64, 64, 64, 0, 0,
-			  mode.dockstartx, 0, mode.dockstarty, 0, 0, 0, 0, 0, 1,
-			  0, 1);
+	bt = ButtonCreate(id, ic, ac, NULL, NULL, 1, 0,
+			  ewin->w, ewin->w, ewin->h, ewin->h,
+			  0, 0, Conf.dock.startx, 0, Conf.dock.starty, 0,
+			  0, 0, 0, 0, 1, 0, 1);
      }
    else
      {
-	bt = CreateButton(id, ic, ac, NULL, NULL, 1, 0, 64, 64, 64, 64, 0, 2046,
-			  0, 0, 0, 1023, 0, 0, 0, 0, 1, 0, 1);
+	bt = ButtonCreate(id, ic, ac, NULL, NULL, 1, 0,
+			  ewin->w, ewin->w, ewin->h, ewin->h,
+			  0, 2046, 0, 0, 0, 1023, 0, 0, 0, 0, 1, 0, 1);
      }
    UngrabX();
 
    if (!bt)
       EDBUG_RETURN_;
 
-   ShowButton(bt);
-   FindEmptySpotForButton(bt, "DOCK_APP_BUTTON", mode.dockdirmode);
+   ButtonShow(bt);
+   ButtonFindEmptySpotFor(bt, "DOCK_APP_BUTTON", Conf.dock.dirmode);
 
-   AddItem(ewin, "EWIN", ewin->client.win, LIST_TYPE_EWIN);
    AddItem(bt, "DOCK_APP_BUTTON", ewin->client.win, LIST_TYPE_BUTTON);
 
-   EmbedWindowIntoButton(bt, ewin->client.win);
+   ButtonEmbedWindow(bt, ewin->win);
+   EMoveResizeWindow(disp, ewin->win, 0, 0, ewin->w, ewin->h);
+   ewin->fixedpos = 1;
    ShowEwin(ewin);
 
    EDBUG_RETURN_;
@@ -71,7 +74,7 @@ DockDestroy(EWin * ewin)
    EDBUG(3, "DockDestroy");
    bt = FindItem(NULL, ewin->client.win, LIST_FINDBY_ID, LIST_TYPE_BUTTON);
    if (bt)
-      DestroyButton(RemoveItem
+      ButtonDestroy(RemoveItem
 		    (NULL, ewin->client.win, LIST_FINDBY_ID, LIST_TYPE_BUTTON));
    EDBUG_RETURN_;
 }
