@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Kim Woelders
+ * Copyright (C) 2003-2004 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -25,11 +25,6 @@
  */
 #include "E.h"
 
-#if DEBUG_EWMH
-#undef EDBUG
-#define EDBUG(a,b) printf(b "\n")
-#endif
-
 #define _ATOM_INIT(atom) atom = XInternAtom(disp, #atom, False); \
     atom_list[atom_count++] = atom
 
@@ -53,7 +48,6 @@
 Atom                E_XA_UTF8_STRING;
 
 /* Window manager info */
-Atom                _NET_WM_NAME;
 Atom                _NET_SUPPORTED;
 Atom                _NET_SUPPORTING_WM_CHECK;
 
@@ -65,6 +59,7 @@ Atom                _NET_CURRENT_DESKTOP;
 Atom                _NET_DESKTOP_VIEWPORT;
 Atom                _NET_WORKAREA;
 Atom                _NET_VIRTUAL_ROOTS;
+Atom                _NET_SHOWING_DESKTOP;
 
 Atom                _NET_ACTIVE_WINDOW;
 Atom                _NET_CLIENT_LIST;
@@ -73,6 +68,10 @@ Atom                _NET_CLIENT_LIST_STACKING;
 /* Misc window ops */
 Atom                _NET_CLOSE_WINDOW;
 
+#if 0				/* Not yet implemented */
+Atom                _NET_MOVERESIZE_WINDOW;
+#endif
+
 /*
  * _NET_WM_MOVERESIZE
  *
@@ -96,6 +95,13 @@ Atom                _NET_WM_MOVERESIZE;
 /*
  * Application Window Properties
  */
+Atom                _NET_WM_NAME;
+Atom                _NET_WM_ICON_NAME;
+
+#if 0				/* Not yet implemented */
+Atom                _NET_WM_VISIBLE_NAME;
+Atom                _NET_WM_VISIBLE_ICON_NAME;
+#endif
 Atom                _NET_WM_DESKTOP;
 
 /* _NET_WM_WINDOW_TYPE (window property) */
@@ -125,16 +131,28 @@ Atom                _NET_WM_STATE_FULLSCREEN;
 Atom                _NET_WM_STATE_ABOVE;
 Atom                _NET_WM_STATE_BELOW;
 
+Atom                _NET_WM_WINDOW_OPACITY;
+
 /* Window state property change actions */
 #define _NET_WM_STATE_REMOVE    0
 #define _NET_WM_STATE_ADD       1
 #define _NET_WM_STATE_TOGGLE    2
 
-/* EWMH flags (somewhat messy) */
-#define NET_WM_FLAG_MAXIMIZED_VERT  0x01
-#define NET_WM_FLAG_MAXIMIZED_HORZ  0x02
+Atom                _NET_WM_STRUT;
 
+#if 0				/* Not yet implemented */
+Atom                _NET_WM_ALLOWED_ACTIONS;
+Atom                _NET_WM_ICON_GEOMETRY;
 Atom                _NET_WM_ICON;
+Atom                _NET_WM_PID;
+Atom                _NET_WM_HANDLED_ICONS;
+
+Atom                _NET_WM_PING;
+#endif
+
+#ifndef ENABLE_HINTS_GNOME
+Atom                _G_WIN_LAUER;
+#endif
 
 /*
  * Set/clear Atom in list
@@ -167,21 +185,6 @@ atom_list_set(Atom * atoms, int size, int *count, Atom atom, int set)
      }
 }
 
-/*
- * Return index of window in list, -1 if not found.
- * Search starts at end (utility to help finding the stacking order).
- */
-static int
-winlist_rindex(Window * wl, int len, Window win)
-{
-   int                 i;
-
-   for (i = len - 1; i >= 0; i--)
-      if (win == wl[i])
-	 break;
-   return i;
-}
-
 /*
  * Initialize EWMH stuff
  */
@@ -193,11 +196,10 @@ EWMH_Init(Window win_wm_check)
 
    EDBUG(6, "EWMH_Init");
 
-   E_XA_UTF8_STRING = XInternAtom(disp, "UTF8_STRING", False);;
+   E_XA_UTF8_STRING = XInternAtom(disp, "UTF8_STRING", False);
 
    atom_count = 0;
 
-   _ATOM_INIT(_NET_WM_NAME);
    _ATOM_INIT(_NET_SUPPORTED);
    _ATOM_INIT(_NET_SUPPORTING_WM_CHECK);
 
@@ -208,6 +210,7 @@ EWMH_Init(Window win_wm_check)
    _ATOM_INIT(_NET_DESKTOP_VIEWPORT);
    _ATOM_INIT(_NET_WORKAREA);
    _ATOM_INIT(_NET_VIRTUAL_ROOTS);
+   _ATOM_INIT(_NET_SHOWING_DESKTOP);
 
    _ATOM_INIT(_NET_ACTIVE_WINDOW);
    _ATOM_INIT(_NET_CLIENT_LIST);
@@ -216,8 +219,9 @@ EWMH_Init(Window win_wm_check)
    _ATOM_INIT(_NET_CLOSE_WINDOW);
    _ATOM_INIT(_NET_WM_MOVERESIZE);
 
+   _ATOM_INIT(_NET_WM_NAME);
+   _ATOM_INIT(_NET_WM_ICON_NAME);
    _ATOM_INIT(_NET_WM_DESKTOP);
-   _ATOM_INIT(_NET_WM_ICON);
 
    _ATOM_INIT(_NET_WM_WINDOW_TYPE);
    _ATOM_INIT(_NET_WM_WINDOW_TYPE_DESKTOP);
@@ -238,18 +242,20 @@ EWMH_Init(Window win_wm_check)
    _ATOM_INIT(_NET_WM_STATE_SKIP_TASKBAR);
    _ATOM_INIT(_NET_WM_STATE_SKIP_PAGER);
    _ATOM_INIT(_NET_WM_STATE_HIDDEN);
-#if 0
    _ATOM_INIT(_NET_WM_STATE_FULLSCREEN);
-#endif
    _ATOM_INIT(_NET_WM_STATE_ABOVE);
    _ATOM_INIT(_NET_WM_STATE_BELOW);
 
-   _ATOM_SET_ATOM(_NET_SUPPORTED, root.win, atom_list, atom_count);
+   _ATOM_INIT(_NET_WM_WINDOW_OPACITY);
+
+   _ATOM_INIT(_NET_WM_STRUT);
+
+   _ATOM_SET_ATOM(_NET_SUPPORTED, VRoot.win, atom_list, atom_count);
 
    /* Set WM info properties */
-   _ATOM_SET_UTF8_STRING(_NET_WM_NAME, root.win, e_wm_name);
+   _ATOM_SET_UTF8_STRING(_NET_WM_NAME, VRoot.win, e_wm_name);
 
-   _ATOM_SET_WINDOW(_NET_SUPPORTING_WM_CHECK, root.win, &win_wm_check, 1);
+   _ATOM_SET_WINDOW(_NET_SUPPORTING_WM_CHECK, VRoot.win, &win_wm_check, 1);
    _ATOM_SET_WINDOW(_NET_SUPPORTING_WM_CHECK, win_wm_check, &win_wm_check, 1);
    _ATOM_SET_UTF8_STRING(_NET_WM_NAME, win_wm_check, e_wm_name);
 
@@ -274,14 +280,14 @@ EWMH_SetDesktopCount(void)
 
    EDBUG(6, "EWMH_SetDesktopCount");
 
-   val = mode.numdesktops;
-   _ATOM_SET_CARD32(_NET_NUMBER_OF_DESKTOPS, root.win, &val, 1);
+   val = Conf.desks.num;
+   _ATOM_SET_CARD32(_NET_NUMBER_OF_DESKTOPS, VRoot.win, &val, 1);
 
-   for (i = 0; i < mode.numdesktops; i++)
+   for (i = 0; i < Conf.desks.num; i++)
      {
 	wl[i] = desks.desk[i].win;
      }
-   _ATOM_SET_WINDOW(_NET_VIRTUAL_ROOTS, root.win, &wl, mode.numdesktops);
+   _ATOM_SET_WINDOW(_NET_VIRTUAL_ROOTS, VRoot.win, &wl, Conf.desks.num);
 
    EDBUG_RETURN_;
 }
@@ -295,10 +301,10 @@ EWMH_SetDesktopNames(void)
    EDBUG(6, "EWMH_SetDesktopNames");
 
    s = buf;
-   for (i = 0; i < mode.numdesktops; i++)
+   for (i = 0; i < Conf.desks.num; i++)
       s += sprintf(s, "Desk-%d", i) + 1;
 
-   _ATOM_SET_UTF8_STRING_LIST(_NET_DESKTOP_NAMES, root.win, buf, s - buf);
+   _ATOM_SET_UTF8_STRING_LIST(_NET_DESKTOP_NAMES, VRoot.win, buf, s - buf);
 
    EDBUG_RETURN_;
 }
@@ -311,9 +317,9 @@ EWMH_SetDesktopSize(void)
 
    EDBUG(6, "EWMH_SetDesktopSize");
    GetAreaSize(&ax, &ay);
-   size[0] = ax * root.w;
-   size[1] = ay * root.h;
-   _ATOM_SET_CARD32(_NET_DESKTOP_GEOMETRY, root.win, &size, 2);
+   size[0] = ax * VRoot.w;
+   size[1] = ay * VRoot.h;
+   _ATOM_SET_CARD32(_NET_DESKTOP_GEOMETRY, VRoot.win, &size, 2);
    EDBUG_RETURN_;
 }
 
@@ -325,18 +331,18 @@ EWMH_SetWorkArea(void)
 
    EDBUG(6, "EWMH_SetWorkArea");
 
-   n_coord = 4 * mode.numdesktops;
+   n_coord = 4 * Conf.desks.num;
    p_coord = Emalloc(n_coord * sizeof(CARD32));
    if (p_coord)
      {
-	for (i = 0; i < mode.numdesktops; i++)
+	for (i = 0; i < Conf.desks.num; i++)
 	  {
 	     p_coord[4 * i] = 0;
 	     p_coord[4 * i + 1] = 0;
-	     p_coord[4 * i + 2] = root.w;
-	     p_coord[4 * i + 3] = root.h;
+	     p_coord[4 * i + 2] = VRoot.w;
+	     p_coord[4 * i + 3] = VRoot.h;
 	  }
-	_ATOM_SET_CARD32(_NET_WORKAREA, root.win, p_coord, n_coord);
+	_ATOM_SET_CARD32(_NET_WORKAREA, VRoot.win, p_coord, n_coord);
 	Efree(p_coord);
      }
    EDBUG_RETURN_;
@@ -349,7 +355,7 @@ EWMH_SetCurrentDesktop(void)
 
    EDBUG(6, "EWMH_SetCurrentDesktop");
    val = desks.current;
-   _ATOM_SET_CARD32(_NET_CURRENT_DESKTOP, root.win, &val, 1);
+   _ATOM_SET_CARD32(_NET_CURRENT_DESKTOP, VRoot.win, &val, 1);
    EDBUG_RETURN_;
 }
 
@@ -360,16 +366,16 @@ EWMH_SetDesktopViewport(void)
    int                 n_coord, i;
 
    EDBUG(6, "EWMH_SetDesktopViewport");
-   n_coord = 2 * mode.numdesktops;
+   n_coord = 2 * Conf.desks.num;
    p_coord = Emalloc(n_coord * sizeof(CARD32));
    if (p_coord)
      {
-	for (i = 0; i < mode.numdesktops; i++)
+	for (i = 0; i < Conf.desks.num; i++)
 	  {
-	     p_coord[2 * i] = desks.desk[i].current_area_x * root.w;
-	     p_coord[2 * i + 1] = desks.desk[i].current_area_y * root.h;
+	     p_coord[2 * i] = desks.desk[i].current_area_x * VRoot.w;
+	     p_coord[2 * i + 1] = desks.desk[i].current_area_y * VRoot.h;
 	  }
-	_ATOM_SET_CARD32(_NET_DESKTOP_VIEWPORT, root.win, p_coord, n_coord);
+	_ATOM_SET_CARD32(_NET_DESKTOP_VIEWPORT, VRoot.win, p_coord, n_coord);
 	Efree(p_coord);
      }
    EDBUG_RETURN_;
@@ -383,74 +389,99 @@ void
 EWMH_SetClientList(void)
 {
    Window             *wl;
-   int                 i, j, k, nwin, num;
+   int                 i, num;
    EWin              **lst;
 
    EDBUG(6, "EWMH_SetClientList");
+
    /* Mapping order */
    lst = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
-   wl = NULL;
-   nwin = 0;
-   if (lst)
+   if (num > 0)
      {
-	wl = Emalloc(sizeof(Window) * num);
+	wl = Emalloc(num * sizeof(Window));
 	for (i = 0; i < num; i++)
-	  {
-	     EWin               *ewin = lst[i];
-
-	     if (ewin->iconified == 4)
-		continue;
-	     wl[nwin++] = ewin->client.win;
-	  }
+	   wl[i] = lst[i]->client.win;
+	_ATOM_SET_WINDOW(_NET_CLIENT_LIST, VRoot.win, wl, num);
+	Efree(wl);
+     }
+   else
+     {
+	_ATOM_SET_WINDOW(_NET_CLIENT_LIST, VRoot.win, NULL, 0);
      }
-   _ATOM_SET_WINDOW(_NET_CLIENT_LIST, root.win, wl, nwin);
    if (lst)
       Efree(lst);
 
-   /*
-    * Stacking order.
-    * We will only bother ourselves with the ones on this desktop.
-    */
-   num = desks.desk[desks.current].num;
-   lst = desks.desk[desks.current].list;
-   for (i = j = 0; i < num; i++)
-     {
-	Window              win = lst[i]->client.win;
-
-	k = winlist_rindex(wl, nwin - j, win);
-	if (k < 0)
-	   continue;
-	/* Swap 'em */
-	wl[k] = wl[nwin - 1 - j];
-	wl[nwin - 1 - j] = win;
-	j++;
-     }
-   _ATOM_SET_WINDOW(_NET_CLIENT_LIST_STACKING, root.win, wl, nwin);
-   if (wl)
-      Efree(wl);
    EDBUG_RETURN_;
 }
 
 void
-EWMH_SetActiveWindow(const EWin * ewin)
+EWMH_SetClientStacking(void)
+{
+   Window             *wl;
+   int                 i, num;
+   EWin               *const *lst;
+
+   EDBUG(6, "EWMH_SetClientStacking");
+
+   /* Stacking order */
+   lst = EwinListGetStacking(&num);
+   if (num > 0)
+     {
+	wl = Emalloc(num * sizeof(Window));
+	for (i = 0; i < num; i++)
+	   wl[i] = lst[num - i - 1]->client.win;
+	_ATOM_SET_WINDOW(_NET_CLIENT_LIST_STACKING, VRoot.win, wl, num);
+	Efree(wl);
+     }
+   else
+     {
+	_ATOM_SET_WINDOW(_NET_CLIENT_LIST_STACKING, VRoot.win, NULL, 0);
+     }
+
+   EDBUG_RETURN_;
+}
+
+void
+EWMH_SetActiveWindow(Window win)
 {
    static Window       win_last_set;
-   Window              win;
 
    EDBUG(6, "EWMH_SetActiveWindow");
-   win = (ewin) ? ewin->client.win : None;
    if (win != win_last_set)
      {
-	_ATOM_SET_WINDOW(_NET_ACTIVE_WINDOW, root.win, &win, 1);
+	_ATOM_SET_WINDOW(_NET_ACTIVE_WINDOW, VRoot.win, &win, 1);
 	win_last_set = win;
      }
    EDBUG_RETURN_;
 }
 
+void
+EWMH_SetShowingDesktop(int on)
+{
+   CARD32              val;
+
+   EDBUG(6, "EWMH_SetShowingDesktop");
+   val = on;
+   _ATOM_SET_CARD32(_NET_SHOWING_DESKTOP, VRoot.win, &val, 1);
+   EDBUG_RETURN_;
+}
+
 /*
  * Functions that set X11-properties from E-window internals
  */
 
+void
+EWMH_SetWindowName(Window win, const char *name)
+{
+   const char         *str;
+
+   EDBUG(6, "EWMH_SetWindowName");
+   str = EstrInt2Enc(name, 1);
+   _ATOM_SET_UTF8_STRING(_NET_WM_NAME, win, str);
+   EstrInt2EncFree(str, 1);
+   EDBUG_RETURN_;
+}
+
 void
 EWMH_SetWindowDesktop(const EWin * ewin)
 {
@@ -483,9 +514,11 @@ EWMH_SetWindowState(const EWin * ewin)
    atom_list_set(atom_list, len, &atom_count, _NET_WM_STATE_HIDDEN,
 		 ewin->iconified || ewin->shaded);
    atom_list_set(atom_list, len, &atom_count, _NET_WM_STATE_MAXIMIZED_VERT,
-		 ewin->ewmh_flags & NET_WM_FLAG_MAXIMIZED_VERT);
+		 ewin->st.maximized_vert);
    atom_list_set(atom_list, len, &atom_count, _NET_WM_STATE_MAXIMIZED_HORZ,
-		 ewin->ewmh_flags & NET_WM_FLAG_MAXIMIZED_HORZ);
+		 ewin->st.maximized_horz);
+   atom_list_set(atom_list, len, &atom_count, _NET_WM_STATE_FULLSCREEN,
+		 ewin->st.fullscreen);
    atom_list_set(atom_list, len, &atom_count, _NET_WM_STATE_SKIP_PAGER,
 		 ewin->skip_ext_pager);
    atom_list_set(atom_list, len, &atom_count, _NET_WM_STATE_ABOVE,
@@ -496,10 +529,65 @@ EWMH_SetWindowState(const EWin * ewin)
    EDBUG_RETURN_;
 }
 
+void
+EWMH_SetWindowOpacity(EWin * ewin, unsigned int opacity)
+{
+   CARD32              val = opacity;
+
+   ewin->props.opacity = opacity;
+   _ATOM_SET_CARD32(_NET_WM_WINDOW_OPACITY, ewin->win, &val, 1);
+}
+
 /*
  * Functions that set E-window internals from X11-properties
  */
 
+void
+EWMH_GetWindowName(EWin * ewin)
+{
+   char               *val;
+   int                 size;
+
+   EDBUG(6, "EWMH_GetWindowName");
+
+   val = AtomGet(ewin->client.win, _NET_WM_NAME, E_XA_UTF8_STRING, &size);
+   if (!val)
+      goto done;
+
+   if (ewin->ewmh.wm_name)
+      Efree(ewin->ewmh.wm_name);
+   ewin->ewmh.wm_name = EstrUtf82Int(val, size);
+
+   Efree(val);
+   EwinChange(ewin, EWIN_CHANGE_NAME);
+
+ done:
+   EDBUG_RETURN_;
+}
+
+void
+EWMH_GetWindowIconName(EWin * ewin)
+{
+   char               *val;
+   int                 size;
+
+   EDBUG(6, "EWMH_GetWindowIconName");
+
+   val = AtomGet(ewin->client.win, _NET_WM_ICON_NAME, E_XA_UTF8_STRING, &size);
+   if (!val)
+      goto done;
+
+   if (ewin->ewmh.wm_icon_name)
+      Efree(ewin->ewmh.wm_icon_name);
+   ewin->ewmh.wm_icon_name = EstrUtf82Int(val, size);
+
+   Efree(val);
+   EwinChange(ewin, EWIN_CHANGE_ICON_NAME);
+
+ done:
+   EDBUG_RETURN_;
+}
+
 void
 EWMH_GetWindowDesktop(EWin * ewin)
 {
@@ -510,7 +598,7 @@ EWMH_GetWindowDesktop(EWin * ewin)
 
    val = AtomGet(ewin->client.win, _NET_WM_DESKTOP, XA_CARDINAL, &size);
    if (!val)
-      goto exit;
+      goto done;
 
    if ((unsigned)val[0] == 0xFFFFFFFF)
      {
@@ -524,8 +612,9 @@ EWMH_GetWindowDesktop(EWin * ewin)
 	ewin->sticky = 0;
      }
    Efree(val);
+   EwinChange(ewin, EWIN_CHANGE_DESKTOP);
 
- exit:
+ done:
    EDBUG_RETURN_;
 }
 
@@ -537,18 +626,17 @@ EWMH_GetWindowState(EWin * ewin)
 
    EDBUG(6, "EWMH_GetWindowState");
 
-   ewin->ewmh_flags = 0;
-
    n_atoms = 0;
    p_atoms = AtomGet(ewin->client.win, _NET_WM_STATE, XA_ATOM, &n_atoms);
    n_atoms /= sizeof(Atom);	/* Silly */
    if (!p_atoms)
-      goto exit;
+      goto done;
 
    /* We must clear/set all according to not present/present */
    ewin->sticky = ewin->shaded = 0;
    ewin->skiptask = ewin->skip_ext_pager = 0;
-   ewin->ewmh_flags = 0;
+   ewin->st.maximized_horz = ewin->st.maximized_vert = 0;
+   ewin->st.fullscreen = 0;
 /* ewin->layer = No ... TBD */
 
    for (i = 0; i < n_atoms; i++)
@@ -565,9 +653,11 @@ EWMH_GetWindowState(EWin * ewin)
 	else if (atom == _NET_WM_STATE_HIDDEN)
 	   ;			/* ewin->iconified = 1; No - WM_STATE does this */
 	else if (atom == _NET_WM_STATE_MAXIMIZED_VERT)
-	   ewin->ewmh_flags |= NET_WM_FLAG_MAXIMIZED_VERT;
+	   ewin->st.maximized_vert = 1;
 	else if (atom == _NET_WM_STATE_MAXIMIZED_HORZ)
-	   ewin->ewmh_flags |= NET_WM_FLAG_MAXIMIZED_HORZ;
+	   ewin->st.maximized_horz = 1;
+	else if (atom == _NET_WM_STATE_FULLSCREEN)
+	   ewin->st.fullscreen = 1;
 	else if (atom == _NET_WM_STATE_ABOVE)
 	   ewin->layer = 6;
 	else if (atom == _NET_WM_STATE_BELOW)
@@ -575,7 +665,7 @@ EWMH_GetWindowState(EWin * ewin)
      }
    Efree(p_atoms);
 
- exit:
+ done:
    EDBUG_RETURN_;
 }
 
@@ -591,23 +681,19 @@ EWMH_GetWindowType(EWin * ewin)
    p_atoms = AtomGet(ewin->client.win, _NET_WM_WINDOW_TYPE, XA_ATOM, &n_atoms);
    n_atoms /= sizeof(Atom);	/* Silly */
    if (!p_atoms)
-      goto exit;
+      goto done;
 
    atom = p_atoms[0];
    if (atom == _NET_WM_WINDOW_TYPE_DESKTOP)
      {
-	Border             *b;
-
 	ewin->layer = 0;
 	ewin->sticky = 1;
+#if 0				/* Should be configurable */
 	ewin->focusclick = 1;
-	ewin->neverraise = 1;
+#endif
 	ewin->skipfocus = 1;
 	ewin->fixedpos = 1;
-	b = (Border *) FindItem("BORDERLESS", 0, LIST_FINDBY_NAME,
-				LIST_TYPE_BORDER);
-	ewin->border_new = 1;
-	SetEwinToBorder(ewin, b);
+	EwinSetBorderByName(ewin, "BORDERLESS", 0);
      }
    else if (atom == _NET_WM_WINDOW_TYPE_DOCK)
      {
@@ -625,7 +711,7 @@ EWMH_GetWindowType(EWin * ewin)
 	ewin->skipfocus = 1;
 	ewin->never_use_area = 1;
      }
-#if 0
+#if 0				/* Not used by E (yet?) */
    else if (atom == _NET_WM_WINDOW_TYPE_TOOLBAR)
      {
      }
@@ -644,7 +730,66 @@ EWMH_GetWindowType(EWin * ewin)
 #endif
    Efree(p_atoms);
 
- exit:
+ done:
+   EDBUG_RETURN_;
+}
+
+static void
+EWMH_GetWindowMisc(EWin * ewin)
+{
+   CARD32             *val;
+   int                 size;
+
+   EDBUG(6, "EWMH_GetWindowMisc");
+
+   val = AtomGet(ewin->client.win, _NET_SUPPORTING_WM_CHECK, XA_WINDOW, &size);
+   if (val)
+     {
+	ewin->props.vroot = 1;
+	Efree(val);
+     }
+}
+
+static void
+EWMH_GetWindowOpacity(EWin * ewin)
+{
+   CARD32             *val;
+   int                 size;
+
+   EDBUG(6, "EWMH_GetWindowOpacity");
+
+   val = AtomGet(ewin->client.win, _NET_WM_WINDOW_OPACITY, XA_CARDINAL, &size);
+   if (val)
+     {
+	ewin->props.opacity = val[0];
+	EWMH_SetWindowOpacity(ewin, val[0]);
+	Efree(val);
+     }
+
+   EDBUG_RETURN_;
+}
+
+static void
+EWMH_GetWindowStrut(EWin * ewin)
+{
+   CARD32             *val;
+   int                 size;
+
+   EDBUG(6, "EWMH_GetWindowStrut");
+
+   val = AtomGet(ewin->client.win, _NET_WM_STRUT, XA_CARDINAL, &size);
+   if (val)
+     {
+	if (size > 4)
+	  {
+	     ewin->strut.left = val[0];
+	     ewin->strut.right = val[1];
+	     ewin->strut.top = val[2];
+	     ewin->strut.bottom = val[3];
+	  }
+	Efree(val);
+     }
+
    EDBUG_RETURN_;
 }
 
@@ -652,10 +797,15 @@ void
 EWMH_GetWindowHints(EWin * ewin)
 {
    EDBUG(6, "EWMH_GetWindowHints");
+   EWMH_GetWindowMisc(ewin);
+   EWMH_GetWindowOpacity(ewin);
+   EWMH_GetWindowName(ewin);
+   EWMH_GetWindowIconName(ewin);
    EWMH_GetWindowDesktop(ewin);
    EWMH_GetWindowState(ewin);
    EWMH_GetWindowType(ewin);
 /*  EWMH_GetWindowIcons(ewin);  TBD */
+   EWMH_GetWindowStrut(ewin);
    EDBUG_RETURN_;
 }
 
@@ -705,12 +855,19 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
    if (event->message_type == _NET_CURRENT_DESKTOP)
      {
 	GotoDesktop(event->data.l[0]);
-	goto exit;
+	goto done;
      }
    else if (event->message_type == _NET_DESKTOP_VIEWPORT)
      {
-	SetCurrentArea(event->data.l[0] / root.w, event->data.l[1] / root.h);
-	goto exit;
+	SetCurrentArea(event->data.l[0] / VRoot.w, event->data.l[1] / VRoot.h);
+	goto done;
+     }
+   else if (event->message_type == _NET_SHOWING_DESKTOP)
+     {
+	Eprintf("EWMH_ProcessClientMessage: _NET_SHOWING_DESKTOP: %ld\n",
+		event->data.l[0]);
+	EWMH_SetShowingDesktop(event->data.l[0]);
+	goto done;
      }
 
    /*
@@ -718,7 +875,7 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
     */
    ewin = FindItem(NULL, event->window, LIST_FINDBY_ID, LIST_TYPE_EWIN);
    if (ewin == NULL)
-      goto exit;
+      goto done;
 
    if (event->message_type == _NET_ACTIVE_WINDOW)
      {
@@ -726,24 +883,24 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
 	   DeIconifyEwin(ewin);
 	RaiseEwin(ewin);
 	if (ewin->shaded)
-	   UnShadeEwin(ewin);
-	FocusToEWin(ewin);
+	   EwinUnShade(ewin);
+	FocusToEWin(ewin, FOCUS_SET);
      }
    else if (event->message_type == _NET_CLOSE_WINDOW)
      {
-	KillEwin(ewin);
+	ActionsCall(ACTION_KILL, ewin, NULL);
      }
    else if (event->message_type == _NET_WM_DESKTOP)
      {
 	if ((unsigned)event->data.l[0] == 0xFFFFFFFF)
 	  {
 	     if (!ewin->sticky)
-		MakeWindowSticky(ewin);
+		EwinStick(ewin);
 	  }
 	else
 	  {
 	     if (ewin->sticky)
-		MakeWindowUnSticky(ewin);
+		EwinUnStick(ewin);
 	     else
 		MoveEwinToDesktop(ewin, event->data.l[0]);
 	  }
@@ -764,18 +921,18 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
 	  {
 	     action = do_set(ewin->sticky, action);
 	     if (action)
-		MakeWindowSticky(ewin);
+		EwinStick(ewin);
 	     else
-		MakeWindowUnSticky(ewin);
+		EwinUnStick(ewin);
 	     ewin->sticky = action;
 	  }
 	else if (atom == _NET_WM_STATE_SHADED)
 	  {
 	     action = do_set(ewin->shaded, action);
 	     if (action)
-		ShadeEwin(ewin);
+		EwinShade(ewin);
 	     else
-		UnShadeEwin(ewin);
+		EwinUnShade(ewin);
 	     ewin->shaded = action;
 	  }
 	else if (atom == _NET_WM_STATE_SKIP_TASKBAR)
@@ -793,67 +950,64 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
 	else if (atom == _NET_WM_STATE_MAXIMIZED_VERT ||
 		 atom == _NET_WM_STATE_MAXIMIZED_HORZ)
 	  {
-	     void                (*func) (EWin *, char *);
-	     int                 maskbits;
+	     void                (*func) (EWin *, const char *);
+	     int                 maxh, maxv;
 
+	     maxh = ewin->st.maximized_horz;
+	     maxv = ewin->st.maximized_vert;
 	     if (atom2 == _NET_WM_STATE_MAXIMIZED_VERT || atom2 == _NET_WM_STATE_MAXIMIZED_HORZ)	/* (ok - ok) */
 	       {
 		  func = MaxSize;
-		  maskbits = NET_WM_FLAG_MAXIMIZED_VERT |
-		     NET_WM_FLAG_MAXIMIZED_HORZ;
+		  maxh = do_set(maxh, action);
+		  maxv = do_set(maxv, action);
 	       }
 	     else if (atom == _NET_WM_STATE_MAXIMIZED_VERT)
 	       {
 		  func = MaxHeight;
-		  maskbits = NET_WM_FLAG_MAXIMIZED_VERT;
+		  maxv = do_set(maxv, action);
 	       }
 	     else
 	       {
 		  func = MaxWidth;
-		  maskbits = NET_WM_FLAG_MAXIMIZED_HORZ;
+		  maxh = do_set(maxh, action);
 	       }
 
-	     if (ewin->ewmh_flags & maskbits)
-	       {
-		  if (action != _NET_WM_STATE_ADD)
-		    {
-		       ewin->ewmh_flags &= ~maskbits;
-		       ewin->toggle = 1;
-		    }
-	       }
+	     if ((ewin->st.maximized_horz == maxh) &&
+		 (ewin->st.maximized_vert == maxv))
+		goto done;
+
+	     if ((ewin->st.maximized_horz && !maxh) ||
+		 (ewin->st.maximized_vert && !maxv))
+		ewin->toggle = 1;
 	     else
-	       {
-		  if (action != _NET_WM_STATE_REMOVE)
-		    {
-		       ewin->ewmh_flags |= maskbits;
-		       ewin->toggle = 0;
-		    }
-	       }
+		ewin->toggle = 0;
+
 	     func(ewin, "available");
-	     RememberImportantInfoForEwin(ewin);
 	     EWMH_SetWindowState(ewin);
-	     ewin->toggle = 0;
 	  }
-#if 0				/* Not yet implemented */
 	else if (atom == _NET_WM_STATE_FULLSCREEN)
 	  {
+	     action = do_set(ewin->st.fullscreen, action);
+	     if (ewin->st.fullscreen == action)
+		goto done;
+
+	     ActionsCall(ACTION_FULLSCREEN, ewin, (action) ? "on" : "off");
 	  }
-#endif
 	else if (atom == _NET_WM_STATE_ABOVE)
 	  {
 	     action = do_set(ewin->layer >= 6, action);
 	     if (action)
-		doSetLayer("6");
+		ActionsCall(ACTION_SET_LAYER, ewin, "6");
 	     else
-		doSetLayer("4");
+		ActionsCall(ACTION_SET_LAYER, ewin, "4");
 	  }
 	else if (atom == _NET_WM_STATE_BELOW)
 	  {
 	     action = do_set(ewin->layer <= 2, action);
 	     if (action)
-		doSetLayer("2");
+		ActionsCall(ACTION_SET_LAYER, ewin, "2");
 	     else
-		doSetLayer("4");
+		ActionsCall(ACTION_SET_LAYER, ewin, "4");
 	  }
      }
    else if (event->message_type == _NET_WM_MOVERESIZE)
@@ -861,15 +1015,15 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
 	switch (event->data.l[2])
 	  {
 	  case _NET_WM_MOVERESIZE_SIZE_KEYBOARD:
-	     doResize(NULL);
+	     /* doResize(NULL); */
 	     break;
 	  case _NET_WM_MOVERESIZE_MOVE_KEYBOARD:
-	     doMove(NULL);
+	     /* doMove(NULL); */
 	     break;
 	  }
      }
 
- exit:
+ done:
    EDBUG_RETURN_;
 }
 
@@ -880,11 +1034,15 @@ void
 EWMH_ProcessPropertyChange(EWin * ewin, Atom atom_change)
 {
    EDBUG(6, "EWMH_ProcessPropertyChange");
-#if 0
-   if (atom_change == _NET_WM_DESKTOP)
-      EWMH_GetWindowDesktop(ewin);
-   else if (atom_change == _NET_WM_STATE)
-      EWMH_GetWindowState(ewin);
-#endif
+
+   if (atom_change == _NET_WM_NAME)
+      EWMH_GetWindowName(ewin);
+   else if (atom_change == _NET_WM_ICON_NAME)
+      EWMH_GetWindowIconName(ewin);
+   else if (atom_change == _NET_WM_STRUT)
+      EWMH_GetWindowStrut(ewin);
+   else if (atom_change == _NET_WM_WINDOW_OPACITY)
+      EWMH_GetWindowOpacity(ewin);
+
    EDBUG_RETURN_;
 }
