@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,16 +21,14 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include <sys/stat.h>
+#include <ctype.h>
+#include <dirent.h>
+#include <pwd.h>
+#include <time.h>
 
-#ifdef __EMX__
-#define chdir	_chdir2
-#define getcwd	_getcwd2
-extern char        *__XOS2RedirRoot(const char *);
-
-#endif
-
-char               *
-FileExtension(char *file)
+const char         *
+FileExtension(const char *file)
 {
    char               *p;
 
@@ -52,13 +50,13 @@ Etmp(char *s)
    EDBUG(9, "Etmp");
    if (!n_calls)
       n_calls = (unsigned long)time(NULL) + (unsigned long)getpid();
-   Esnprintf(s, 1024, "%s/TMP_%Xl", UserEDir(), n_calls);
+   Esnprintf(s, 1024, "%s/TMP_%lX", EDirUser(), n_calls);
    n_calls++;
    EDBUG_RETURN_;
 }
 
 void
-md(char *s)
+E_md(const char *s)
 {
    EDBUG(9, "md");
    if ((!s) || (!*s))
@@ -68,7 +66,7 @@ md(char *s)
 }
 
 int
-exists(char *s)
+exists(const char *s)
 {
    struct stat         st;
 
@@ -81,7 +79,7 @@ exists(char *s)
 }
 
 void
-mkdirs(char *s)
+mkdirs(const char *s)
 {
    char                ss[FILEPATH_LEN_MAX];
    int                 i, ii;
@@ -95,7 +93,7 @@ mkdirs(char *s)
 	if (s[i] == '/')
 	  {
 	     if (!exists(ss))
-		md(ss);
+		E_md(ss);
 	     else if (!isdir(ss))
 		return;
 	  }
@@ -104,7 +102,7 @@ mkdirs(char *s)
 }
 
 int
-isfile(char *s)
+isfile(const char *s)
 {
    struct stat         st;
 
@@ -119,7 +117,7 @@ isfile(char *s)
 }
 
 int
-isdir(char *s)
+isdir(const char *s)
 {
    struct stat         st;
 
@@ -133,8 +131,14 @@ isdir(char *s)
    EDBUG_RETURN(0);
 }
 
+int
+isabspath(const char *path)
+{
+   return path[0] == '/';
+}
+
 char              **
-ls(char *dir, int *num)
+E_ls(const char *dir, int *num)
 {
    int                 i, dirlen;
    int                 done = 0;
@@ -172,7 +176,7 @@ ls(char *dir, int *num)
 	   break;
 	if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, "..")))
 	  {
-	     names[i] = duplicate(dp->d_name);
+	     names[i] = Estrdup(dp->d_name);
 	     i++;
 	  }
      }
@@ -215,7 +219,7 @@ freestrlist(char **l, int num)
 }
 
 void
-rm(char *s)
+E_rm(const char *s)
 {
    EDBUG(9, "rm");
    if ((!s) || (!*s))
@@ -225,7 +229,7 @@ rm(char *s)
 }
 
 void
-mv(char *s, char *ss)
+E_mv(const char *s, const char *ss)
 {
    EDBUG(9, "mv");
    if ((!s) || (!ss) || (!*s) || (!*ss))
@@ -235,7 +239,7 @@ mv(char *s, char *ss)
 }
 
 void
-cp(char *s, char *ss)
+E_cp(const char *s, const char *ss)
 {
    int                 i;
    FILE               *f, *ff;
@@ -264,7 +268,7 @@ cp(char *s, char *ss)
 }
 
 time_t
-moddate(char *s)
+moddate(const char *s)
 {
    struct stat         st;
 
@@ -279,7 +283,7 @@ moddate(char *s)
 }
 
 int
-filesize(char *s)
+filesize(const char *s)
 {
    struct stat         st;
 
@@ -292,9 +296,8 @@ filesize(char *s)
 }
 
 int
-fileinode(char *s)
+fileinode(const char *s)
 {
-#ifndef __EMX__
    struct stat         st;
 
    EDBUG(9, "filesize");
@@ -303,13 +306,10 @@ fileinode(char *s)
    if (stat(s, &st) < 0)
       EDBUG_RETURN(0);
    EDBUG_RETURN((int)st.st_ino);
-#else
-   EDBUG_RETURN(0);
-#endif
 }
 
 int
-filedev(char *s)
+filedev(const char *s)
 {
    struct stat         st;
 
@@ -322,7 +322,7 @@ filedev(char *s)
 }
 
 void
-cd(char *s)
+E_cd(const char *s)
 {
    EDBUG(9, "cd");
    if ((!s) || (!*s))
@@ -339,12 +339,12 @@ cwd(void)
 
    EDBUG(9, "cwd");
    getcwd(ss, FILEPATH_LEN_MAX);
-   s = duplicate(ss);
+   s = Estrdup(ss);
    EDBUG_RETURN(s);
 }
 
 int
-permissions(char *s)
+permissions(const char *s)
 {
    struct stat         st;
 
@@ -356,8 +356,9 @@ permissions(char *s)
    EDBUG_RETURN(st.st_mode);
 }
 
+#if 0				/* Not used */
 int
-owner(char *s)
+owner(const char *s)
 {
    struct stat         st;
 
@@ -370,7 +371,7 @@ owner(char *s)
 }
 
 int
-group(char *s)
+group(const char *s)
 {
    struct stat         st;
 
@@ -381,13 +382,12 @@ group(char *s)
       EDBUG_RETURN(0);
    EDBUG_RETURN(st.st_gid);
 }
+#endif
 
 char               *
 username(int uid)
 {
    char               *s;
-
-#ifndef __EMX__
    static int          usr_uid = -1;
    static char        *usr_s = NULL;
    struct passwd      *pwd;
@@ -396,28 +396,22 @@ username(int uid)
    if (usr_uid < 0)
       usr_uid = getuid();
    if ((uid == usr_uid) && (usr_s))
-      EDBUG_RETURN(duplicate(usr_s));
+      EDBUG_RETURN(Estrdup(usr_s));
    pwd = getpwuid(uid);
    if (pwd)
      {
-	s = duplicate(pwd->pw_name);
+	s = Estrdup(pwd->pw_name);
 	if (uid == usr_uid)
-	   usr_s = duplicate(s);
+	   usr_s = Estrdup(s);
 	EDBUG_RETURN(s);
      }
-#else
-   if ((s = getenv("USER")) != NULL)
-      EDBUG_RETURN(duplicate(s));
-#endif
-   EDBUG_RETURN(duplicate("unknown"));
+   EDBUG_RETURN(Estrdup("unknown"));
 }
 
 char               *
 homedir(int uid)
 {
    char               *s;
-
-#ifndef __EMX__
    static int          usr_uid = -1;
    static char        *usr_s = NULL;
    struct passwd      *pwd;
@@ -427,30 +421,23 @@ homedir(int uid)
       usr_uid = getuid();
    if ((uid == usr_uid) && (usr_s))
      {
-	EDBUG_RETURN(duplicate(usr_s));
+	EDBUG_RETURN(Estrdup(usr_s));
      }
    pwd = getpwuid(uid);
    if (pwd)
      {
-	s = duplicate(pwd->pw_dir);
+	s = Estrdup(pwd->pw_dir);
 	if (uid == usr_uid)
-	   usr_s = duplicate(s);
+	   usr_s = Estrdup(s);
 	EDBUG_RETURN(s);
      }
-#else
-   if ((s = getenv("HOME")) != NULL)
-      EDBUG_RETURN(_fnslashify(duplicate(s)));
-   if ((s = getenv("TMP")) != NULL)
-      EDBUG_RETURN(_fnslashify(duplicate(s)));
-#endif
-   EDBUG_RETURN(duplicate
-		((getenv("TMPDIR") == NULL) ? "/tmp" : getenv("TMPDIR")));
+   EDBUG_RETURN(Estrdup((getenv("TMPDIR") == NULL) ?
+			"/tmp" : getenv("TMPDIR")));
 }
 
 char               *
 usershell(int uid)
 {
-#ifndef __EMX__
    char               *s;
    static int          usr_uid = -1;
    static char        *usr_s = NULL;
@@ -460,29 +447,26 @@ usershell(int uid)
    if (usr_uid < 0)
       usr_uid = getuid();
    if ((uid == usr_uid) && (usr_s))
-      return duplicate(usr_s);
+      return Estrdup(usr_s);
    pwd = getpwuid(uid);
    if (pwd)
      {
 	if (!pwd->pw_shell)
-	   EDBUG_RETURN(duplicate("/bin/sh"));
+	   EDBUG_RETURN(Estrdup("/bin/sh"));
 	if (strlen(pwd->pw_shell) < 1)
-	   EDBUG_RETURN(duplicate("/bin/sh"));
+	   EDBUG_RETURN(Estrdup("/bin/sh"));
 	if (!(canexec(pwd->pw_shell)))
-	   EDBUG_RETURN(duplicate("/bin/sh"));
-	s = duplicate(pwd->pw_shell);
+	   EDBUG_RETURN(Estrdup("/bin/sh"));
+	s = Estrdup(pwd->pw_shell);
 	if (uid == usr_uid)
-	   usr_s = duplicate(s);
+	   usr_s = Estrdup(s);
 	EDBUG_RETURN(s);
      }
-   EDBUG_RETURN(duplicate("/bin/sh"));
-#else
-   EDBUG_RETURN(duplicate("sh.exe"));
-#endif
+   EDBUG_RETURN(Estrdup("/bin/sh"));
 }
 
-char               *
-atword(char *s, int num)
+const char         *
+atword(const char *s, int num)
 {
    int                 cnt, i;
 
@@ -508,8 +492,8 @@ atword(char *s, int num)
    EDBUG_RETURN(NULL);
 }
 
-char               *
-atchar(char *s, char c)
+const char         *
+atchar(const char *s, char c)
 {
    int                 i;
 
@@ -597,10 +581,10 @@ getword(char *s, int num)
 }
 
 void
-word(char *s, int num, char *wd)
+word(const char *s, int num, char *wd)
 {
    int                 cnt, i;
-   char               *start, *finish, *ss, *w;
+   const char         *start, *finish, *ss;
 
    EDBUG(9, "word");
    if (!s)
@@ -617,7 +601,6 @@ word(char *s, int num, char *wd)
    start = NULL;
    finish = NULL;
    ss = NULL;
-   w = wd;
 
    while (s[i])
      {
@@ -749,30 +732,30 @@ fword(char *s, int num, char *wd)
 }
 
 char               *
-field(char *s, int field)
+field(char *s, int fieldno)
 {
    char                buf[4096];
 
    EDBUG(9, "field");
    buf[0] = 0;
-   fword(s, field + 1, buf);
+   fword(s, fieldno + 1, buf);
    if (buf[0])
      {
 	if ((!strcmp(buf, "NULL")) || (!strcmp(buf, "(null)")))
 	   EDBUG_RETURN(NULL);
-	EDBUG_RETURN(duplicate(buf));
+	EDBUG_RETURN(Estrdup(buf));
      }
    EDBUG_RETURN(NULL);
 }
 
 int
-fillfield(char *s, int field, char *buf)
+fillfield(char *s, int fieldno, char *buf)
 {
    EDBUG(9, "fillfield");
    if (!buf)
       EDBUG_RETURN(0);
    buf[0] = 0;
-   fword(s, field + 1, buf);
+   fword(s, fieldno + 1, buf);
    if (buf[0])
      {
 	if ((!strcmp(buf, "NULL")) || (!strcmp(buf, "(null)")))
@@ -786,7 +769,7 @@ fillfield(char *s, int field, char *buf)
 }
 
 int
-canread(char *s)
+canread(const char *s)
 {
    EDBUG(9, "canread");
    if ((!s) || (!*s))
@@ -799,7 +782,7 @@ canread(char *s)
 }
 
 int
-canwrite(char *s)
+canwrite(const char *s)
 {
    EDBUG(9, "canwrite");
    if ((!s) || (!*s))
@@ -812,7 +795,7 @@ canwrite(char *s)
 }
 
 int
-canexec(char *s)
+canexec(const char *s)
 {
    EDBUG(9, "canexec");
    if ((!s) || (!*s))
@@ -825,116 +808,52 @@ canexec(char *s)
 }
 
 char               *
-fileof(char *s)
+fileof(const char *path)
 {
-   char                ss[1024];
-   int                 i, p1, p2;
+   const char         *s1, *s2;
 
-   EDBUG(9, "fileof");
-   i = 0;
-   p1 = -1;
-   p2 = -1;
-   for (i = strlen(s) - 1; i >= 0; i--)
-     {
-	if ((s[i] == '.') && (p2 < 0) && (p1 < 0))
-	   p2 = i - 1;
-	if ((s[i] == '/') && (p1 < 0))
-	   p1 = i + 1;
-     }
-   if (p2 < 0)
-      p2 = strlen(s) - 1;
-   if (p1 < 0)
-      p1 = 0;
-   if (p2 <= 0)
-      EDBUG_RETURN(duplicate(""));
-   for (i = 0; i <= (p2 - p1); i++)
-      ss[i] = s[p1 + i];
-   ss[i] = 0;
-   EDBUG_RETURN(duplicate(ss));
+   s1 = strrchr(path, '/');
+   s1 = (s1) ? s1 + 1 : path;
+   s2 = strrchr(s1, '.');
+   if (!s2)
+      return Estrdup(s1);
+
+   return Estrndup(s1, s2 - s1);
 }
 
 char               *
-fullfileof(char *s)
+fullfileof(const char *path)
 {
-   char                ss[1024];
-   int                 i, p1, p2;
+   const char         *s;
 
-   EDBUG(9, "fullfileof");
-   i = 0;
-   p1 = -1;
-   for (i = strlen(s) - 1; i >= 0; i--)
-     {
-	if ((s[i] == '/') && (p1 < 0))
-	   p1 = i + 1;
-     }
-   if (p1 < 0)
-      p1 = 0;
-   p2 = strlen(s);
-   for (i = 0; i < (p2 - p1); i++)
-      ss[i] = s[p1 + i];
-   ss[i] = 0;
-   EDBUG_RETURN(duplicate(ss));
+   s = strrchr(path, '/');
+   return Estrdup((s) ? s + 1 : path);
 }
 
 char               *
-pathtoexec(char *file)
+pathtoexec(const char *file)
 {
    char               *p, *cp, *ep;
    char               *s;
    int                 len, exelen;
 
    EDBUG(9, "pathtoexec");
-#ifndef __EMX__
-   if (file[0] == '/')
-#else
-   if (_fnisabs(file))
-#endif
+
+   if (isabspath(file))
      {
 	if (canexec(file))
-	   EDBUG_RETURN(duplicate(file));
-#ifdef __EMX__
-	len = strlen(file);
-	s = Emalloc(len + 5);
-	strcpy(s, file);
-	strcat(s, ".cmd");
-	if (canexec(s))
-	   EDBUG_RETURN(s);
-	strcpy(s + len, ".exe");
-	if (canexec(s))
-	   EDBUG_RETURN(s);
-	Efree(s);
-	if (file[0] != '/' && file[0] != '\\')
-	   EDBUG_RETURN(NULL);
-	file = __XOS2RedirRoot(file);
-	if (canexec(file))
-	   EDBUG_RETURN(duplicate(file));
-	len = strlen(file);
-	s = Emalloc(len + 5);
-	strcpy(s, file);
-	strcat(s, ".cmd");
-	if (canexec(s))
-	   EDBUG_RETURN(s);
-	strcpy(s + len, ".exe");
-	if (canexec(s))
-	   EDBUG_RETURN(s);
-	Efree(s);
-#endif
+	   EDBUG_RETURN(Estrdup(file));
 	EDBUG_RETURN(NULL);
      }
    p = getenv("PATH");
    if (!p)
-      EDBUG_RETURN(duplicate(file));
+      EDBUG_RETURN(Estrdup(file));
    if (!file)
       EDBUG_RETURN(NULL);
-#ifndef __EMX__
+
    cp = p;
    exelen = strlen(file);
    while ((ep = strchr(cp, ':')))
-#else
-   cp = p = _fnslashify(strdup(p));
-   exelen = strlen(file) + 4;
-   while ((ep = strchr(cp, ';')))
-#endif
      {
 	len = ep - cp;
 	s = Emalloc(len + 1);
@@ -943,32 +862,10 @@ pathtoexec(char *file)
 	     strncpy(s, cp, len);
 	     s[len] = 0;
 	     s = Erealloc(s, len + 2 + exelen);
-#ifdef __EMX__
-	     if (s[len - 1] != '/')
-#endif
-		strcat(s, "/");
+	     strcat(s, "/");
 	     strcat(s, file);
 	     if (canexec(s))
-#ifndef __EMX__
 		EDBUG_RETURN(s);
-#else
-	       {
-		  free(p);
-		  EDBUG_RETURN(s);
-	       }
-	     strcat(s, ".cmd");
-	     if (canexec(s))
-	       {
-		  free(p);
-		  EDBUG_RETURN(s);
-	       }
-	     strcpy(s + strlen(s) - 3, "exe");
-	     if (canexec(s))
-	       {
-		  free(p);
-		  EDBUG_RETURN(s);
-	       }
-#endif
 	     Efree(s);
 	  }
 	cp = ep + 1;
@@ -980,10 +877,7 @@ pathtoexec(char *file)
 	strncpy(s, cp, len);
 	s[len] = 0;
 	s = Erealloc(s, len + 2 + exelen);
-#ifdef __EMX__
-	if (s[len - 1] != '/')
-#endif
-	   strcat(s, "/");
+	strcat(s, "/");
 	strcat(s, file);
 	if (canexec(s))
 	   EDBUG_RETURN(s);
@@ -993,35 +887,27 @@ pathtoexec(char *file)
 }
 
 char               *
-pathtofile(char *file)
+pathtofile(const char *file)
 {
    char               *p, *cp, *ep;
    char               *s;
    int                 len, exelen;
 
    EDBUG(9, "pathtofile");
-#ifndef __EMX__
-   if (file[0] == '/')
-#else
-   if (_fnisabs(file))
-#endif
+
+   if (isabspath(file))
      {
 	if (exists(file))
-	   EDBUG_RETURN(duplicate(file));
+	   EDBUG_RETURN(Estrdup(file));
      }
    p = getenv("PATH");
    if (!p)
-      EDBUG_RETURN(duplicate(file));
+      EDBUG_RETURN(Estrdup(file));
    if (!file)
       EDBUG_RETURN(NULL);
    cp = p;
    exelen = strlen(file);
-#ifndef __EMX__
    while ((ep = strchr(cp, ':')))
-#else
-   cp = p = _fnslashify(strdup(p));
-   while ((ep = strchr(cp, ';')))
-#endif
      {
 	len = ep - cp;
 	s = Emalloc(len + 1);
@@ -1030,20 +916,10 @@ pathtofile(char *file)
 	     strncpy(s, cp, len);
 	     s[len] = 0;
 	     s = Erealloc(s, len + 2 + exelen);
-#ifdef __EMX__
-	     if (s[len - 1] != '/')
-#endif
-		strcat(s, "/");
+	     strcat(s, "/");
 	     strcat(s, file);
 	     if (exists(s))
-#ifndef __EMX__
 		EDBUG_RETURN(s);
-#else
-	       {
-		  free(p);
-		  EDBUG_RETURN(s);
-	       }
-#endif
 	     Efree(s);
 	  }
 	cp = ep + 1;
@@ -1055,40 +931,26 @@ pathtofile(char *file)
 	strncpy(s, cp, len);
 	s[len] = 0;
 	s = Erealloc(s, len + 2 + exelen);
-#ifdef __EMX__
-	if (s[len - 1] != '/')
-#endif
-	   strcat(s, "/");
+	strcat(s, "/");
 	strcat(s, file);
 	if (exists(s))
-#ifndef __EMX__
 	   EDBUG_RETURN(s);
-#else
-	  {
-	     free(p);
-	     EDBUG_RETURN(s);
-	  }
-#endif
 	Efree(s);
      }
-#ifdef __EMX__
-   free(p);
-#endif
    EDBUG_RETURN(NULL);
 }
 
 int
 findLocalizedFile(char *fname)
 {
-#ifndef __EMX__
    char               *tmp, *lang, *p[3];
    int                 i;
 
    if (!(lang = setlocale(LC_MESSAGES, NULL)))
       return 0;
 
-   tmp = strdup(fname);
-   lang = strdup(lang);		/* lang may be in static space, thus it must
+   tmp = Estrdup(fname);
+   lang = Estrdup(lang);	/* lang may be in static space, thus it must
 				 * * * be duplicated before we change it below */
    p[0] = lang + strlen(lang);
    p[1] = strchr(lang, '.');
@@ -1111,7 +973,17 @@ findLocalizedFile(char *fname)
    strcpy(fname, tmp);
    free(tmp);
    free(lang);
-#endif
 
    return 0;
 }
+
+#if 0				/* Not used */
+void
+rmrf(const char *path)
+{
+   char                s[FILEPATH_LEN_MAX];
+
+   Esnprintf(s, sizeof(s), "/bin/rm -rf %s", path);
+   system(s);
+}
+#endif
