@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,6 +21,7 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include <math.h>
 
 /* Someone may do this right one day, but for now - kill'em */
 #define ENABLE_FX_INFO 0
@@ -31,8 +32,8 @@
 
 typedef struct _fxhandler
 {
-   char               *name;
-   void                (*init_func) (char *name);
+   const char         *name;
+   void                (*init_func) (const char *name);
    void                (*desk_func) (void);
    void                (*quit_func) (void);
    void                (*pause_func) (void);
@@ -41,19 +42,19 @@ typedef struct _fxhandler
 }
 FXHandler;
 
-void                FX_Ripple_Init(char *name);
+void                FX_Ripple_Init(const char *name);
 void                FX_Ripple_Desk(void);
 void                FX_Ripple_Quit(void);
 void                FX_Ripple_Pause(void);
-void                FX_Raindrops_Init(char *name);
+void                FX_Raindrops_Init(const char *name);
 void                FX_Raindrops_Desk(void);
 void                FX_Raindrops_Quit(void);
 void                FX_Raindrops_Pause(void);
-void                FX_Waves_Init(char *name);
+void                FX_Waves_Init(const char *name);
 void                FX_Waves_Desk(void);
 void                FX_Waves_Quit(void);
 void                FX_Waves_Pause(void);
-void                FX_ImageSpinner_Init(char *name);
+void                FX_ImageSpinner_Init(const char *name);
 void                FX_ImageSpinner_Desk(void);
 void                FX_ImageSpinner_Quit(void);
 void                FX_ImageSpinner_Pause(void);
@@ -81,7 +82,7 @@ static FXHandler    fx_handlers[] = {
 static FXHandler   *
 FX_Find(const char *name)
 {
-   int                 i;
+   unsigned int        i;
 
    for (i = 0; i < N_FX_HANDLERS; i++)
       if (!strcmp(fx_handlers[i].name, name))
@@ -106,7 +107,7 @@ FX_Op(const char *name, int fx_op)
 	   break;
       do_start:
 	if (fxh->init_func)
-	   (*(fxh->init_func)) (name);
+	   fxh->init_func(name);
 	fxh->in_use = 1;
 	break;
 
@@ -128,10 +129,16 @@ FX_Op(const char *name, int fx_op)
      }
 }
 
-void
+#if 0
+
+/*
+ e  Doesn't look like this is ever used, commented out for now
+ * --Mandrake
+ */
+static void
 FX_Activate(char *effect)
 {
-   int                 i;
+   unsigned int        i;
 
    for (i = 0; i < N_FX_HANDLERS; i++)
      {
@@ -140,17 +147,18 @@ FX_Activate(char *effect)
 	     if (!fx_handlers[i].in_use)
 	       {
 		  fx_handlers[i].in_use = 1;
-		  (*(fx_handlers[i].init_func)) (effect);
+		  fx_handlers[i].init_func(effect);
 	       }
 	  }
      }
    return;
 }
+#endif
 
 void
 FX_DeskChange(void)
 {
-   int                 i;
+   unsigned int        i;
 
    for (i = 0; i < N_FX_HANDLERS; i++)
      {
@@ -165,7 +173,7 @@ FX_DeskChange(void)
 void
 FX_Pause(void)
 {
-   int                 i;
+   unsigned int        i;
 
    for (i = 0; i < N_FX_HANDLERS; i++)
      {
@@ -174,13 +182,13 @@ FX_Pause(void)
 	     if (fx_handlers[i].paused)
 	       {
 		  if (fx_handlers[i].pause_func)
-		     (*(fx_handlers[i].pause_func)) ();
+		     fx_handlers[i].pause_func();
 		  fx_handlers[i].paused = 1;
 	       }
 	     else
 	       {
 		  if (fx_handlers[i].pause_func)
-		     (*(fx_handlers[i].pause_func)) ();
+		     fx_handlers[i].pause_func();
 		  fx_handlers[i].paused = 0;
 	       }
 	  }
@@ -190,7 +198,7 @@ FX_Pause(void)
 char              **
 FX_Active(int *num)
 {
-   int                 i;
+   unsigned int        i;
    char              **list = NULL;
 
    *num = 0;
@@ -201,16 +209,16 @@ FX_Active(int *num)
 	     (*num)++;
 	     list = Erealloc(list, sizeof(char *) * (*num));
 
-	     list[(*num) - 1] = duplicate(fx_handlers[i].name);
+	     list[(*num) - 1] = Estrdup(fx_handlers[i].name);
 	  }
      }
    return list;
 }
 
 int
-FX_IsOn(char *effect)
+FX_IsOn(const char *effect)
 {
-   int                 i;
+   unsigned int        i;
 
    for (i = 0; i < N_FX_HANDLERS; i++)
      {
@@ -220,7 +228,6 @@ FX_IsOn(char *effect)
 	  }
      }
    return 0;
-
 }
 
 /****************************** RIPPLES *************************************/
@@ -237,15 +244,15 @@ FX_ripple_info(void)
    static char         before = 0;
 
    if (!before)
-      DIALOG_OK(_("Starting up Ripples FX..."),
-		_("\n" "You have just started the Ripples Effect.\n" "\n"
-		  "If you look closely on your desktop background, and if it\n"
-		  "doesn't have a solid colour (i.e. has a background texture or\n"
-		  "image), you will see a pool of water at the bottom of your\n"
-		  "screen that reflects everything above it and \"ripples\".\n"
-		  "\n"
-		  "To disable this effect just select this option again to toggle\n"
-		  "it off.\n"));
+      DialogOK(_("Starting up Ripples FX..."),
+	       _("\n" "You have just started the Ripples Effect.\n" "\n"
+		 "If you look closely on your desktop background, and if it\n"
+		 "doesn't have a solid colour (i.e. has a background texture or\n"
+		 "image), you will see a pool of water at the bottom of your\n"
+		 "screen that reflects everything above it and \"ripples\".\n"
+		 "\n"
+		 "To disable this effect just select this option again to toggle\n"
+		 "it off.\n"));
    before = 1;
 #endif
 }
@@ -262,14 +269,10 @@ FX_ripple_timeout(int val, void *data)
 	XGCValues           gcv;
 
 	fx_ripple_win = desks.desk[desks.current].win;
-	if ((desks.current == 0) && (prImlibData))
-	   fx_ripple_above =
-	      ECreatePixmap(disp, fx_ripple_win, root.w, fx_ripple_waterh * 2,
-			    prImlibData->x.depth);
-	else
-	   fx_ripple_above =
-	      ECreatePixmap(disp, fx_ripple_win, root.w, fx_ripple_waterh * 2,
-			    pImlibData->x.depth);
+
+	fx_ripple_above =
+	   ECreatePixmap(disp, fx_ripple_win, VRoot.w, fx_ripple_waterh * 2,
+			 GetWinDepth(fx_ripple_win));
 	if (gc)
 	   XFreeGC(disp, gc);
 	if (gc1)
@@ -282,7 +285,7 @@ FX_ripple_timeout(int val, void *data)
      }
    if (fx_ripple_count == 0)
       XCopyArea(disp, fx_ripple_win, fx_ripple_above, gc, 0,
-		root.h - (fx_ripple_waterh * 3), root.w, fx_ripple_waterh * 2,
+		VRoot.h - (fx_ripple_waterh * 3), VRoot.w, fx_ripple_waterh * 2,
 		0, 0);
    fx_ripple_count++;
    if (fx_ripple_count > 32)
@@ -308,8 +311,8 @@ FX_ripple_timeout(int val, void *data)
 	yy = (fx_ripple_waterh * 2) - yoff;
 	aa = p * p * 64 + inch;
 	off = (int)(sin(aa) * 10 * (1 - p));
-	XCopyArea(disp, fx_ripple_above, fx_ripple_win, gc1, 0, yy, root.w, 1,
-		  off, root.h - fx_ripple_waterh + y);
+	XCopyArea(disp, fx_ripple_above, fx_ripple_win, gc1, 0, yy, VRoot.w, 1,
+		  off, VRoot.h - fx_ripple_waterh + y);
      }
    DoIn("FX_RIPPLE_TIMEOUT", 0.066, FX_ripple_timeout, 0, NULL);
    return;
@@ -318,10 +321,11 @@ FX_ripple_timeout(int val, void *data)
 }
 
 void
-FX_Ripple_Init(char *name)
+FX_Ripple_Init(const char *name)
 {
    fx_ripple_count = 0;
    DoIn("FX_RIPPLE_TIMEOUT", 0.066, FX_ripple_timeout, 0, NULL);
+   return;
    name = NULL;
 }
 
@@ -337,7 +341,7 @@ void
 FX_Ripple_Quit(void)
 {
    RemoveTimerEvent("FX_RIPPLE_TIMEOUT");
-   XClearArea(disp, fx_ripple_win, 0, root.h - fx_ripple_waterh, root.w,
+   XClearArea(disp, fx_ripple_win, 0, VRoot.h - fx_ripple_waterh, VRoot.w,
 	      fx_ripple_waterh, False);
 }
 
@@ -388,16 +392,16 @@ FX_raindrops_info(void)
    static char         before = 0;
 
    if (!before)
-      DIALOG_OK(_("Starting up Raindrops FX..."),
-		_("\n" "You have just started the Raindrops Effect.\n"
-		  "\n"
-		  "If you look closely on your desktop background, and if it\n"
-		  "doesn't have a solid colour (i.e. has a background texture or\n"
-		  "image), you will see \"raindrops\" hit the background and\n"
-		  "make little splashes. This Effect can be VERY CPU intensive.\n"
-		  "\n"
-		  "To disable this effect just select this option again to toggle\n"
-		  "it off.\n"));
+      DialogOK(_("Starting up Raindrops FX..."),
+	       _("\n" "You have just started the Raindrops Effect.\n"
+		 "\n"
+		 "If you look closely on your desktop background, and if it\n"
+		 "doesn't have a solid colour (i.e. has a background texture or\n"
+		 "image), you will see \"raindrops\" hit the background and\n"
+		 "make little splashes. This Effect can be VERY CPU intensive.\n"
+		 "\n"
+		 "To disable this effect just select this option again to toggle\n"
+		 "it off.\n"));
    before = 1;
 #endif
 }
@@ -416,38 +420,6 @@ FX_raindrops_timeout(int val, void *data)
      {
 	XGCValues           gcv;
 
-	if (!pImlibData->x.shm)
-	  {
-	     DIALOG_OK(_("Unable to display raindrops"),
-		       _("\n"
-			 "Enlightenment is unable to display raindrops on this\n"
-			 "display because Shared memory is not available on this\n"
-			 "X-Server.\n" "\n"
-			 "This may be due to Enlightenment being a remote client\n"
-			 "running over the network, a MIT-SHM incapable X-server,\n"
-			 "having run out of SHM ID's on the system or Shared\n"
-			 "Memory support being turned off in Imlib\n" "\n"
-			 "You may correct this by either running `imlib_config'\n"
-			 "or copying the system imrc (/usr/etc/imrc) to ~/.imrc\n"
-			 "and editing it, enabling shared memory.\n" "\n"));
-	     return;
-	  }
-	if (!pImlibData->x.shmp)
-	  {
-	     DIALOG_OK(_("Unable to display raindrops"),
-		       _("\n"
-			 "Enlightenment is unable to display raindrops on this\n"
-			 "display because shared pixmaps are not available on this\n"
-			 "X-Server.\n" "\n"
-			 "This may be due to either the X-Server not implementing\n"
-			 "shared pixmaps, or shared pixmaps being disabled in\n"
-			 "Imlib's configuration.\n" "\n"
-			 "You may correct this by either running `imlib_config'\n"
-			 "or copying the system imrc (/usr/etc/imrc) to ~/.imrc\n"
-			 "and editing it, enabling shared pixmaps.\n" "\n"));
-	     return;
-	  }
-
 	FX_raindrops_info();
 
 	if (first)
@@ -513,16 +485,20 @@ FX_raindrops_timeout(int val, void *data)
 		  intersect = 0;
 		  for (j = 0; j < fx_raindrops_number; j++)
 		    {
-		       fx_raindrops[i].x = rand() % (root.w - fx_raindrop_size);
-		       fx_raindrops[i].y = rand() % (root.h - fx_raindrop_size);
+		       fx_raindrops[i].x =
+			  rand() % (VRoot.w - fx_raindrop_size);
+		       fx_raindrops[i].y =
+			  rand() % (VRoot.h - fx_raindrop_size);
 		       if (fx_raindrops[i].x < 0)
 			  fx_raindrops[i].x = 0;
-		       else if (fx_raindrops[i].x > (root.w - fx_raindrop_size))
-			  fx_raindrops[i].x = root.w - fx_raindrop_size;
+		       else if (fx_raindrops[i].x >
+				(VRoot.w - fx_raindrop_size))
+			  fx_raindrops[i].x = VRoot.w - fx_raindrop_size;
 		       if (fx_raindrops[i].y < 0)
 			  fx_raindrops[i].y = 0;
-		       else if (fx_raindrops[i].y > (root.h - fx_raindrop_size))
-			  fx_raindrops[i].y = root.h - fx_raindrop_size;
+		       else if (fx_raindrops[i].y >
+				(VRoot.h - fx_raindrop_size))
+			  fx_raindrops[i].y = VRoot.h - fx_raindrop_size;
 		       if (i != j)
 			 {
 			    if (((fx_raindrops[i].x >= fx_raindrops[j].x)
@@ -626,7 +602,7 @@ FX_raindrops_timeout(int val, void *data)
 }
 
 void
-FX_Raindrops_Init(char *name)
+FX_Raindrops_Init(const char *name)
 {
    int                 i;
 
@@ -634,10 +610,11 @@ FX_Raindrops_Init(char *name)
    for (i = 0; i < fx_raindrops_number; i++)
      {
 	fx_raindrops[i].count = rand() % fx_raindrop_duration;
-	fx_raindrops[i].x = rand() % (root.w - fx_raindrop_size);
-	fx_raindrops[i].y = rand() % (root.h - fx_raindrop_size);
+	fx_raindrops[i].x = rand() % (VRoot.w - fx_raindrop_size);
+	fx_raindrops[i].y = rand() % (VRoot.h - fx_raindrop_size);
      }
    DoIn("FX_RAINDROPS_TIMEOUT", 0.066, FX_raindrops_timeout, 0, NULL);
+   return;
    name = NULL;
 }
 
@@ -706,15 +683,15 @@ FX_Wave_info(void)
    static char         before = 0;
 
    if (!before)
-      DIALOG_OK(_("Starting up Waves FX..."),
-		_("\n" "You have just started the Waves Effect.\n" "\n"
-		  "If you look closely on your desktop background, and if it\n"
-		  "doesn't have a solid colour (i.e. has a background texture or\n"
-		  "image), you will see a pool of water at the bottom of your\n"
-		  "screen that reflects everything above it and \"waves\".\n"
-		  "\n"
-		  "To disable this effect just select this option again to toggle\n"
-		  "it off.\n"));
+      DialogOK(_("Starting up Waves FX..."),
+	       _("\n" "You have just started the Waves Effect.\n" "\n"
+		 "If you look closely on your desktop background, and if it\n"
+		 "doesn't have a solid colour (i.e. has a background texture or\n"
+		 "image), you will see a pool of water at the bottom of your\n"
+		 "screen that reflects everything above it and \"waves\".\n"
+		 "\n"
+		 "To disable this effect just select this option again to toggle\n"
+		 "it off.\n"));
    before = 1;
 #endif
 }
@@ -735,14 +712,10 @@ FX_Wave_timeout(int val, void *data)
 	XGCValues           gcv;
 
 	fx_wave_win = desks.desk[desks.current].win;
-	if ((desks.current == 0) && (prImlibData))
-	   fx_wave_above =
-	      XCreatePixmap(disp, fx_wave_win, root.w, FX_WAVE_WATERH * 2,
-			    prImlibData->x.depth);
-	else
-	   fx_wave_above =
-	      XCreatePixmap(disp, fx_wave_win, root.w, FX_WAVE_WATERH * 2,
-			    pImlibData->x.depth);
+
+	fx_wave_above =
+	   XCreatePixmap(disp, fx_wave_win, VRoot.w, FX_WAVE_WATERH * 2,
+			 GetWinDepth(fx_wave_win));
 	if (gc)
 	   XFreeGC(disp, gc);
 	if (gc1)
@@ -758,8 +731,8 @@ FX_Wave_timeout(int val, void *data)
    if (fx_wave_count == 0)
      {
 	XCopyArea(disp, fx_wave_win, fx_wave_above, gc, 0,
-		  root.h - (FX_WAVE_WATERH * 3), root.w, FX_WAVE_WATERH * 2, 0,
-		  0);
+		  VRoot.h - (FX_WAVE_WATERH * 3), VRoot.w, FX_WAVE_WATERH * 2,
+		  0, 0);
      }
 
    /* Increment and roll the counter */
@@ -784,8 +757,8 @@ FX_Wave_timeout(int val, void *data)
    if (fx_wave_count == 0)
      {
 	XCopyArea(disp, fx_wave_above, fx_wave_win, gc1, 0,
-		  root.h - FX_WAVE_GRABH, root.w, FX_WAVE_DEPTH * 2, 0,
-		  root.h - FX_WAVE_GRABH);
+		  VRoot.h - FX_WAVE_GRABH, VRoot.w, FX_WAVE_DEPTH * 2, 0,
+		  VRoot.h - FX_WAVE_GRABH);
      }
 
    /* Go through the bottom couple (FX_WAVE_WATERH) lines of the window */
@@ -808,7 +781,7 @@ FX_Wave_timeout(int val, void *data)
 	incx2 = incx;
 
 	/* Go through the width of the screen, in block sizes */
-	for (x = 0; x < root.w; x += FX_WAVE_WATERW)
+	for (x = 0; x < VRoot.w; x += FX_WAVE_WATERW)
 	  {
 	     /* Variables */
 	     int                 sx;
@@ -825,7 +798,7 @@ FX_Wave_timeout(int val, void *data)
 	     /* Display this block */
 	     XCopyArea(disp, fx_wave_above, fx_wave_win, gc1, x, yy,	/* x, y */
 		       FX_WAVE_WATERW, 1,	/* w, h */
-		       off + x, root.h - FX_WAVE_WATERH + y + sx	/* dx, dy */
+		       off + x, VRoot.h - FX_WAVE_WATERH + y + sx	/* dx, dy */
 		);
 	  }
      }
@@ -842,10 +815,11 @@ FX_Wave_timeout(int val, void *data)
 }
 
 void
-FX_Waves_Init(char *name)
+FX_Waves_Init(const char *name)
 {
    fx_wave_count = 0;
    DoIn("FX_WAVE_TIMEOUT", 0.066, FX_Wave_timeout, 0, NULL);
+   return;
    name = NULL;
 }
 
@@ -861,7 +835,7 @@ void
 FX_Waves_Quit(void)
 {
    RemoveTimerEvent("FX_WAVE_TIMEOUT");
-   XClearArea(disp, fx_wave_win, 0, root.h - FX_WAVE_WATERH, root.w,
+   XClearArea(disp, fx_wave_win, 0, VRoot.h - FX_WAVE_WATERH, VRoot.w,
 	      FX_WAVE_WATERH, False);
 }
 
@@ -888,7 +862,6 @@ FX_Waves_Pause(void)
 
 static Window       fx_imagespinner_win = 0;
 static int          fx_imagespinner_count = 3;
-static ImlibData   *fx_imagespinner_imd = NULL;
 static char        *fx_imagespinner_params = NULL;
 
 static void
@@ -898,11 +871,11 @@ FX_imagespinner_info(void)
    static char         before = 0;
 
    if (!before)
-      DIALOG_OK(_("Starting up imagespinners FX..."),
-		_("\n" "You have just started the imagespinners Effect.\n"
-		  "\n"
-		  "To disable this effect just select this option again to toggle\n"
-		  "it off.\n"));
+      DialogOK(_("Starting up imagespinners FX..."),
+	       _("\n" "You have just started the imagespinners Effect.\n"
+		 "\n"
+		 "To disable this effect just select this option again to toggle\n"
+		 "it off.\n"));
    before = 1;
 #endif
 }
@@ -915,13 +888,9 @@ FX_imagespinner_timeout(int val, void *data)
    if (!fx_imagespinner_win)
      {
 	fx_imagespinner_win = desks.desk[desks.current].win;
-	if ((desks.current == 0) && (prImlibData))
-	   fx_imagespinner_imd = prImlibData;
-	else
-	   fx_imagespinner_imd = pImlibData;
-
 	FX_imagespinner_info();
      }
+
 /* do stuff here */
    string = getword(fx_imagespinner_params, fx_imagespinner_count);
    if (!string)
@@ -929,24 +898,26 @@ FX_imagespinner_timeout(int val, void *data)
 	fx_imagespinner_count = 3;
 	string = getword(fx_imagespinner_params, fx_imagespinner_count);
      }
+
    fx_imagespinner_count++;
    if (string)
      {
-	ImlibImage         *im;
+	Imlib_Image        *im;
 
-	im = ELoadImageImlibData(fx_imagespinner_imd, string);
+	im = ELoadImage(string);
 	if (im)
 	  {
 	     int                 x, y, w, h;
 
-	     w = im->rgb_width;
-	     h = im->rgb_height;
+	     imlib_context_set_image(im);
+	     w = imlib_image_get_width();
+	     h = imlib_image_get_height();
 	     sscanf(fx_imagespinner_params, "%*s %i %i ", &x, &y);
-	     x = ((root.w * x) >> 10) - ((w * x) >> 10);
-	     y = ((root.h * y) >> 10) - ((h * y) >> 10);
-	     Imlib_paste_image(fx_imagespinner_imd, im, fx_imagespinner_win, x,
-			       y, w, h);
-	     Imlib_destroy_image(fx_imagespinner_imd, im);
+	     x = ((VRoot.w * x) >> 10) - ((w * x) >> 10);
+	     y = ((VRoot.h * y) >> 10) - ((h * y) >> 10);
+	     imlib_context_set_drawable(fx_imagespinner_win);
+	     imlib_render_image_on_drawable_at_size(x, y, w, h);
+	     imlib_free_image();
 	  }
 	Efree(string);
      }
@@ -958,28 +929,24 @@ FX_imagespinner_timeout(int val, void *data)
 }
 
 void
-FX_ImageSpinner_Init(char *name)
+FX_ImageSpinner_Init(const char *name)
 {
    fx_imagespinner_count = 3;
    DoIn("FX_IMAGESPINNER_TIMEOUT", 0.066, FX_imagespinner_timeout, 0, NULL);
-   fx_imagespinner_params = duplicate(name);
+   fx_imagespinner_params = Estrdup(name);
 }
 
 void
 FX_ImageSpinner_Desk(void)
 {
    fx_imagespinner_win = desks.desk[desks.current].win;
-   if ((desks.current == 0) && (prImlibData))
-      fx_imagespinner_imd = prImlibData;
-   else
-      fx_imagespinner_imd = pImlibData;
 }
 
 void
 FX_ImageSpinner_Quit(void)
 {
    RemoveTimerEvent("FX_IMAGESPINNER_TIMEOUT");
-   XClearArea(disp, fx_imagespinner_win, 0, 0, root.w, root.h, False);
+   XClearArea(disp, fx_imagespinner_win, 0, 0, VRoot.w, VRoot.h, False);
    if (fx_imagespinner_params)
       Efree(fx_imagespinner_params);
    fx_imagespinner_params = NULL;
