@@ -1,6 +1,5 @@
-
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -35,7 +34,7 @@ GrabActionKey(Action * a)
    if (a->anymodifier)
      {
 	mod = AnyModifier;
-	XGrabKey(disp, a->key, mod, root.win, False, GrabModeAsync,
+	XGrabKey(disp, a->key, mod, VRoot.win, False, GrabModeAsync,
 		 GrabModeAsync);
      }
    else
@@ -44,7 +43,7 @@ GrabActionKey(Action * a)
 
 	/* grab the key even if locks are on or not */
 	for (i = 0; i < 8; i++)
-	   XGrabKey(disp, a->key, mod | mask_mod_combos[i], root.win, False,
+	   XGrabKey(disp, a->key, mod | mask_mod_combos[i], VRoot.win, False,
 		    GrabModeAsync, GrabModeAsync);
      }
    EDBUG_RETURN_;
@@ -62,7 +61,7 @@ UnGrabActionKey(Action * a)
    if (a->anymodifier)
      {
 	mod = AnyModifier;
-	XUngrabKey(disp, a->key, mod, root.win);
+	XUngrabKey(disp, a->key, mod, VRoot.win);
      }
    else
      {
@@ -70,68 +69,72 @@ UnGrabActionKey(Action * a)
 
 	/* ungrab the key even if locks are on or not */
 	for (i = 0; i < 8; i++)
-	   XUngrabKey(disp, a->key, mod | mask_mod_combos[i], root.win);
+	   XUngrabKey(disp, a->key, mod | mask_mod_combos[i], VRoot.win);
      }
    EDBUG_RETURN_;
 }
 
 void
-GrabTheButtons(Window win)
+GrabButtonsSet(Window win, unsigned int csr)
 {
-   EDBUG(4, "GrabTheButtons");
-   if (mode.click_focus_grabbed)
-      EDBUG_RETURN_;
+   if (Mode.grabs.pointer_grab_active)
+      return;
+
    XGrabPointer(disp, win, True, ButtonPressMask | ButtonReleaseMask,
-		GrabModeAsync, GrabModeAsync, None, None, CurrentTime);
-   grab_window = win;
-   mode.click_focus_grabbed = 1;
-   EDBUG_RETURN_;
+		GrabModeAsync, GrabModeAsync, None, ECsrGet(csr), CurrentTime);
+
+   Mode.grabs.pointer_grab_window = win;
+   Mode.grabs.pointer_grab_active = 1;
 }
 
 int
-GrabThePointer(Window win)
+GrabPointerSet(Window win, unsigned int csr, int confine)
 {
    int                 ret;
+   Window              confine_to = (confine) ? win : None;
 
-   EDBUG(4, "GrabThePointer");
-   if (mode.click_focus_grabbed)
-      EDBUG_RETURN(1);
-   ret =
-      XGrabPointer(disp, win, True,
-		   ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
-		   ButtonMotionMask | EnterWindowMask | LeaveWindowMask,
-		   GrabModeAsync, GrabModeAsync, None, None, CurrentTime);
-   grab_window = win;
-   mode.click_focus_grabbed = 1;
-   EDBUG_RETURN(ret);
-}
+   if (Mode.grabs.pointer_grab_active)
+      return 1;
 
-int
-GrabConfineThePointer(Window win)
-{
-   int                 ret;
+   ret = XGrabPointer(disp, win, True,
+		      ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
+		      ButtonMotionMask | EnterWindowMask | LeaveWindowMask,
+		      GrabModeAsync, GrabModeAsync, confine_to, ECsrGet(csr),
+		      CurrentTime);
 
-   EDBUG(4, "GrabThePointer");
-   if (mode.click_focus_grabbed)
-      EDBUG_RETURN(1);
-   ret =
-      XGrabPointer(disp, win, True,
-		   ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
-		   ButtonMotionMask | EnterWindowMask | LeaveWindowMask,
-		   GrabModeAsync, GrabModeAsync, win, None, CurrentTime);
-   grab_window = win;
-   mode.click_focus_grabbed = 1;
-   EDBUG_RETURN(ret);
+   Mode.grabs.pointer_grab_window = win;
+   Mode.grabs.pointer_grab_active = 1;
+
+   return ret;
 }
 
 void
-UnGrabTheButtons()
+GrabPointerRelease(void)
 {
-   EDBUG(4, "UnGrabTheButtons");
-   if (!mode.click_focus_grabbed)
-      EDBUG_RETURN_;
+   if (!Mode.grabs.pointer_grab_active)
+      return;
+
    XUngrabPointer(disp, CurrentTime);
-   mode.click_focus_grabbed = 0;
-   grab_window = 0;
-   EDBUG_RETURN_;
+
+   Mode.grabs.pointer_grab_active = 0;
+   Mode.grabs.pointer_grab_window = None;
+}
+
+void
+GrabButtonSet(unsigned int button, unsigned int modifiers, Window win,
+	      unsigned int event_mask, unsigned int csr, int confine)
+{
+   Bool                owner_events = False;
+   int                 pointer_mode = GrabModeSync;
+   int                 keyboard_mode = GrabModeAsync;
+   Window              confine_to = (confine) ? win : None;
+
+   XGrabButton(disp, button, modifiers, win, owner_events, event_mask,
+	       pointer_mode, keyboard_mode, confine_to, ECsrGet(csr));
+}
+
+void
+GrabButtonRelease(unsigned int button, unsigned int modifiers, Window win)
+{
+   XUngrabButton(disp, button, modifiers, win);
 }
