@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -23,30 +23,36 @@
 #include "E.h"
 #include "timestamp.h"
 #include <sys/utsname.h>
+#include <signal.h>
+#include <time.h>
 
 static void
 runDocBrowser(void)
 {
-
    char                file[FILEPATH_LEN_MAX];
 
+   Esnprintf(file, sizeof(file), "%s/edox", EDirBin());
+   if (!canexec(file))
+      return;
+   Esnprintf(file, sizeof(file), "%s/E-docs", EDirRoot());
+   if (!canread(file))
+      return;
+
    if (fork())
       EDBUG_RETURN_;
 
-   Esnprintf(file, sizeof(file), "exec %s/dox %s/E-docs", ENLIGHTENMENT_BIN,
-	     ENLIGHTENMENT_ROOT);
-   execl(usershell(getuid()), usershell(getuid()), "-c", (char *)file, NULL);
-   exit(0);
+   Esnprintf(file, sizeof(file), "exec %s/edox %s/E-docs",
+	     EDirBin(), EDirRoot());
 
+   execl(usershell(getuid()), usershell(getuid()), "-c", (char *)file, NULL);
+
+   exit(0);
 }
 
 int
 main(int argc, char **argv)
 {
-   int                 i, num;
-   Button            **blst;
-   Background         *bg;
-   ECursor            *ec = NULL;
+   int                 i;
    struct utsname      ubuf;
    char               *str;
 
@@ -54,52 +60,41 @@ main(int argc, char **argv)
     * proceeds into the primary event loop at the end.
     */
 
-   single_screen_mode = 0;
-/*  unsetenv("LD_PRELOAD"); */
+   /* Init state variable struct */
+   memset(&Mode, 0, sizeof(EMode));
+   Mode.mode = MODE_NONE;
+   Mode.wm.startup = 1;
+   Mode.move.check = 1;
 
-/* Part of gettext stuff */
-
-   setlocale(LC_ALL, "");
-   bindtextdomain(PACKAGE, LOCALEDIR);
-   textdomain(PACKAGE);
-
-/* End of gettext stuff */
-
-#ifdef DEBUG
-   call_level = 0;
-   debug_level = 0;
    str = getenv("EDBUG");
    if (str)
-      debug_level = atoi(str);
-#endif
+      Mode.debug = atoi(str);
    str = getenv("EDBUG_FLAGS");
    if (str)
-      debug_flags = strtoul(str, NULL, 0);
+      EventDebugInit(str);
+   str = getenv("EDBUG_COREDUMP");
+   if (str)
+      Mode.wm.coredump = 1;
 
    EDBUG(1, "main");
 
    /* go head and set up the internal data lists that enlightenment
     * uses for finding everything
     */
-   lists = Emalloc(sizeof(List) * LIST_TYPE_COUNT);
-   lists = memset(lists, 0, (sizeof(List) * LIST_TYPE_COUNT));
+   lists = Ecalloc(LIST_TYPE_COUNT, sizeof(List));
 
-   /* Set up all the text bits that belong on the GSOD */
-   AssignTitleText(_("Enlightenment Message Dialog"));
-   AssignIgnoreText(_("Ignore this"));
-   AssignRestartText(_("Restart Enlightenment"));
-   AssignExitText(_("Quit Enlightenment"));
-
-   /* We'll set up what the buttons do now, too */
-   AssignRestartFunction(doExit, "restart");
-   AssignExitFunction(doExit, NULL);
    srand(time(NULL));
 
    if (!uname(&ubuf))
-      e_machine_name = duplicate(ubuf.nodename);
+      e_machine_name = Estrdup(ubuf.nodename);
    if (!e_machine_name)
-      e_machine_name = duplicate("localhost");
-   command = duplicate(argv[0]);
+      e_machine_name = Estrdup("localhost");
+
+   /* Initialise internationalisation */
+   LangInit();
+
+   SetSMProgName(argv[0]);
+
    themepath[0] = 0;
    {
       int                 j = 0;
@@ -117,11 +112,11 @@ main(int argc, char **argv)
 	     }
 	   else if ((!strcmp("-econfdir", argv[j])) && (argc - j > 1))
 	     {
-		SetEDir(argv[++j]);
+		EDirUserSet(argv[++j]);
 	     }
 	   else if ((!strcmp("-ecachedir", argv[j])) && (argc - j > 1))
 	     {
-		SetCacheDir(argv[++j]);
+		EDirUserCacheSet(argv[++j]);
 	     }
 	   else if ((!strcmp("-display", argv[j])) && (argc - j > 1))
 	     {
@@ -129,7 +124,7 @@ main(int argc, char **argv)
 	     }
 	   else if (!strcmp("-single", argv[j]))
 	     {
-		single_screen_mode = 1;
+		Mode.wm.single = 1;
 	     }
 	   else if ((!strcmp("-smid", argv[j])) && (argc - j > 1))
 	     {
@@ -155,31 +150,39 @@ main(int argc, char **argv)
 	     {
 		no_overwrite = 1;
 	     }
-	   else if ((!strcmp("-help", argv[j]))
-		    || (!strcmp("--help", argv[j]))
-		    || (!strcmp("-h", argv[j])) || (!strcmp("-?", argv[j])))
+	   else if ((!strcmp("-window", argv[j])) && (argc - j > 1))
 	     {
-		printf("enlightenment options:                      \n"
-		       "\t-theme /path/to/theme                     \n"
+		sscanf(argv[++j], "%dx%d", &VRoot.w, &VRoot.h);
+		Mode.wm.window = 1;
+		Mode.wm.single = 1;
+	     }
+	   else if ((!strcmp("-h", argv[j])) || (!strcmp("-help", argv[j])) ||
+		    (!strcmp("-?", argv[j])) || (!strcmp("--help", argv[j])))
+	     {
+		printf("enlightenment options:\n"
+		       "\t-display display_name\n"
+		       "\t-ecachedir /path/to/cached/dir\n"
 		       "\t-econfdir /path/to/.enlightenment/conf/dir\n"
-		       "\t-ecachedir /path/to/cached/dir            \n"
-		       "\t[-smid | -clientId | --sm-client-id] id   \n"
-		       "\t-smfile file                              \n"
-		       "\t-ext_init_win window_id                   \n"
-		       "\t-no_overwrite                             \n"
-		       "\t[-v | -version | --version]               \n"
-		       "\t-display display_name                     \n");
+		       "\t-ext_init_win window_id\n"
+		       "\t-no_overwrite\n"
+		       "\t-smfile file\n"
+		       "\t[-smid | -clientId | --sm-client-id] id\n"
+		       "\t-theme /path/to/theme\n"
+		       "\t[-v | -verbose]\n" "\t[-V | -version | --version]\n");
 		exit(0);
 	     }
-	   else if ((!strcmp("-v", argv[j]))
-		    || (!strcmp("-version", argv[j]))
-		    || (!strcmp("--version", argv[j]))
-		    || (!strcmp("-v", argv[j])))
+	   else if ((!strcmp("-V", argv[j])) ||
+		    (!strcmp("-version", argv[j])) ||
+		    (!strcmp("--version", argv[j])))
 	     {
 		printf(_("Enlightenment Version: %s\nLast updated on: %s\n"),
 		       ENLIGHTENMENT_VERSION, E_CHECKOUT_DATE);
 		exit(0);
 	     }
+	   else if ((!strcmp("-v", argv[j])) || (!strcmp("-verbose", argv[j])))
+	     {
+		Mode.debug++;
+	     }
 	}
 
       /* Set a default location for the "previous session" data when
@@ -197,11 +200,7 @@ main(int argc, char **argv)
 	if (file)
 	  {
 	     s[0] = 0;
-#ifndef __EMX__
 	     f = fopen(file, "r");
-#else
-	     f = fopen(file, "rt");
-#endif
 	     if (f)
 	       {
 		  if (fscanf(f, "%4000s", s) < 1)
@@ -217,7 +216,7 @@ main(int argc, char **argv)
 	  {
 	     char               *def;
 
-	     def = GetDefaultTheme();
+	     def = ThemeGetDefault();
 	     if (def)
 	       {
 		  Esnprintf(themepath, sizeof(themepath), "%s", def);
@@ -228,168 +227,160 @@ main(int argc, char **argv)
    SetSMUserThemePath(themepath);
 
    /* run most of the setup */
+   AlertInit();			/* Set up all the text bits that belong on the GSOD */
    SignalsSetup();
-   SetupX();
+   SetupX();			/* This is where the we fork per screen */
    BlumFlimFrub();
    ZoomInit();
    SetupDirs();
    InitDesktopBgs();
-   GotoDesktop(0);
    CommsSetup();
    CommsFindCommsWindow();
-   GrabX();
    LoadGroups();
    LoadSnapInfo();
+
+   GrabX();
    MapUnmap(0);
+   UngrabX();
 
    /* make all of our fallback classes */
    SetupFallbackClasses();
-   UngrabX();
-   /* We'll set up what the buttons do now, too */
-   /* again?  why are we doing this twice? */
-   AssignRestartFunction((*doExit), duplicate("restart"));
-   AssignExitFunction((*EExit), (void *)1);
+
    desks.desk[0].viewable = 0;
    /* now we're going to load the configuration/theme */
    LoadEConfig(themepath);
+   SetAreaSize(Conf.areas.nx, Conf.areas.ny);
+   TransparencySet(Conf.theme.transparency);
 
    desks.desk[0].viewable = 1;
    RefreshDesktop(0);
-   if (mode.sound)
+   if (Conf.sound)
      {
-	ApplySclass(FindItem
-		    ("SOUND_STARTUP", 0, LIST_FINDBY_NAME, LIST_TYPE_SCLASS));
-	DestroySclass(RemoveItem
-		      ("SOUND_STARTUP", 0, LIST_FINDBY_NAME, LIST_TYPE_SCLASS));
+	SoundPlay("SOUND_STARTUP");
+	SoundFree("SOUND_STARTUP");
      }
+
    /* toss down the dragbar and related */
    InitDesktopControls();
+
    /* then draw all the buttons that belong on the desktop */
-   blst = (Button **) ListItemTypeID(&num, LIST_TYPE_BUTTON, 0);
-   if (blst)
-     {
-	for (i = 0; i < num; i++)
-	  {
-	     if ((!blst[i]->internal) && (blst[i]->default_show))
-		ShowButton(blst[i]);
-	  }
-	Efree(blst);
-     }
+   ShowDesktopButtons();
+
    HintsInit();
    SessionInit();
    ShowDesktopControls();
    CheckEvent();
-   /* retreive stuff from last time we were loaded if we're restarting */
-   ICCCM_GetMainEInfo();
+
    SetupEnv();
-   if (mode.mapslide)
+
+   if (Conf.mapslide)
       CreateStartupDisplay(0);
-   MapUnmap(1);
-   /* set some more hints */
-   HintsSetDesktopViewport();
-   desks.current = 0;
-   /* Set up the internal pagers */
+
+   /* Set up the iconboxes and pagers */
    IB_Setup();
-   if (mode.show_pagers)
+   if (Conf.pagers.enable)
      {
-	mode.show_pagers = 0;
-	queue_up = 0;
+	Conf.pagers.enable = 0;
+	Mode.queue_up = 0;
 	EnableAllPagers();
-	queue_up = DRAW_QUEUE_ENABLE;
+	Mode.queue_up = DRAW_QUEUE_ENABLE;
      }
-   if (getpid() == master_pid && init_win_ext)
+
+   MapUnmap(1);
+
+   /* Kill the E process owning the "init window" */
+   if (Mode.wm.master && init_win_ext)
      {
+	if (EventDebug(EDBUG_TYPE_SESSION))
+	   Eprintf("Kill init window %#lx\n", init_win_ext);
 	XKillClient(disp, init_win_ext);
 	init_win_ext = 0;
      }
-   HintsSetClientList();
-
-#if ENABLE_KDE
-   /* start up any kde crap we might need to start up */
-   if (mode.kde_support)
-      KDE_Init();
-#endif
 
    /* sync just to make sure */
    XSync(disp, False);
-   queue_up = DRAW_QUEUE_ENABLE;
+   Mode.queue_up = DRAW_QUEUE_ENABLE;
 
-   /* hello!  we don't have a resizemode of 5! */
-   if (mode.resizemode == 5)
-      mode.resizemode = 0;
    /* of course, we have to set the cursors */
-   ec = FindItem("DEFAULT", 0, LIST_FINDBY_NAME, LIST_TYPE_ECURSOR);
-   if (ec)
-     {
-	ApplyECursor(root.win, ec);
-	ec->ref_count++;
-	ec->inroot = 1;
-     }
+   ECursorsInit();
+   ECsrApply(ECSR_ROOT, VRoot.win);
 
-   mode.startup = 0;
+#if 0				/* AAA: Why here? */
+   Mode.wm.startup = 0;
+#endif
+   Mode.wm.save_ok = Mode.wm.master;
    /* ok - paranoia - save current settings to disk */
-   if (root.scr == 0)
+   if (VRoot.scr == 0)
       autosave();
+
    /* let's make sure we set this up and go to our desk anyways */
-   ICCCM_GetMainEInfo();
    GotoDesktop(desks.current);
-   if (desks.current < (mode.numdesktops - 1))
+#if 0				/* Why? */
+   if (desks.current < (Conf.desks.num - 1))
      {
 	char                ps = 0;
 
-	if (!mode.mapslide)
+	if (!Conf.mapslide)
 	  {
-	     ps = desks.slidein;
-	     desks.slidein = 0;
+	     ps = Conf.desks.slidein;
+	     Conf.desks.slidein = 0;
 	  }
 	GotoDesktop(desks.current + 1);
 	GotoDesktop(desks.current - 1);
-	if (!mode.mapslide)
-	   desks.slidein = ps;
+	if (!Conf.mapslide)
+	   Conf.desks.slidein = ps;
      }
    else if (desks.current > 0)
      {
 	char                ps = 0;
 
-	if (!mode.mapslide)
+	if (!Conf.mapslide)
 	  {
-	     ps = desks.slidein;
-	     desks.slidein = 0;
+	     ps = Conf.desks.slidein;
+	     Conf.desks.slidein = 0;
 	  }
 	GotoDesktop(desks.current - 1);
 	GotoDesktop(desks.current + 1);
-	if (!mode.mapslide)
-	   desks.slidein = ps;
+	if (!Conf.mapslide)
+	   Conf.desks.slidein = ps;
      }
+#endif
    XSync(disp, False);
+
    /* if we didn't have an external window piped to us, we'll do some stuff */
-   if (!mode.mapslide)
+   if (!Conf.mapslide)
       CreateStartupDisplay(0);
-   if ((bg =
-	RemoveItem("STARTUP_BACKGROUND_SIDEWAYS", 0, LIST_FINDBY_NAME,
-		   LIST_TYPE_BACKGROUND)))
-      FreeDesktopBG(bg);
-   if ((bg =
-	RemoveItem("STARTUP_BACKGROUND", 0, LIST_FINDBY_NAME,
-		   LIST_TYPE_BACKGROUND)))
-      FreeDesktopBG(bg);
+
+   BackgroundDestroyByName("STARTUP_BACKGROUND_SIDEWAYS");
+   BackgroundDestroyByName("STARTUP_BACKGROUND");
+
 #ifdef SIGCONT
    for (i = 0; i < child_count; i++)
       kill(e_children[i], SIGCONT);
 #endif
 
    SetupUserInitialization();
-   if (mode.firsttime)
+   if (Mode.firsttime)
       runDocBrowser();
 
-   mode.startup = 1;
+#if 0				/* Not if AAA */
+   Mode.wm.startup = 1;
+#endif
    SpawnSnappedCmds();
-   mode.startup = 0;
+   Mode.wm.startup = 0;
+
+   ThemeBadDialog();
+
+   BackgroundsInit();		/* Start the background accounting */
+
+   /* Update pagers */
+   for (i = 0; i < Conf.desks.num; i++)
+      RedrawPagersForDesktop(i, 0);
 
-   BadThemeDialog();
    /* The primary event loop */
    for (;;)
       WaitEvent();
+
    /* Of course, we should NEVER get to this point */
    EDBUG_RETURN(0);
 }
