@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -23,9 +23,23 @@
 #include "E.h"
 #include <errno.h>
 #include <fcntl.h>
-#include <unistd.h>
+#include <signal.h>
 #include <sys/time.h>
 
+#ifndef DEFAULT_SH_PATH
+#ifdef __sgi
+/*
+ * It appears that SGI (at least IRIX 6.4) uses ksh as their sh, and it
+ * seems to run in restricted mode, so things like restart fail miserably.
+ * Let's use csh instead
+ * -KDT 07/31/98
+ */
+#define DEFAULT_SH_PATH "/sbin/csh"
+#else
+#define DEFAULT_SH_PATH "/bin/sh"
+#endif
+#endif
+
 #ifdef _LIBC
 #include <stdint.h>
 #define gettimeofday __gettimeofday
@@ -38,6 +52,9 @@ typedef unsigned long big_type;
 
 #endif
 
+static char        *command;
+static int          sm_fd = -1;
+
 /* Generate a unique temporary file name from TEMPLATE.
  * The last six characters of TEMPLATE must be "XXXXXX";
  * they are replaced with a string that makes the filename unique.
@@ -46,14 +63,8 @@ int
 Emkstemp(char *template)
 {
    static const char   letters[]
-#ifndef __EMX__
       = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 
-#else
-      = "abcdefghijklmnopqrstuvwxyz0123456789";
-
-#endif
-
    static big_type     value;
    struct timeval      tv;
    char               *XXXXXX;
@@ -122,14 +133,17 @@ static char        *
 default_save_prefix(void)
 {
    static char        *def_prefix = NULL;
+   char                s[1024];
 
-   if (!def_prefix)
-     {
-	char                s[1024];
+   if (def_prefix)
+      return def_prefix;
+
+   if (Mode.wm.window)
+      Esnprintf(s, sizeof(s), "%s/...e_session-window", EDirUser());
+   else
+      Esnprintf(s, sizeof(s), "%s/...e_session-XXXXXX", EDirUser());
+   def_prefix = Estrdup(s);
 
-	Esnprintf(s, sizeof(s), "%s/...e_session-XXXXXX", UserEDir());
-	def_prefix = duplicate(s);
-     }
    return def_prefix;
 }
 
@@ -155,9 +169,15 @@ Match;
 Match              *matches = NULL;
 
 void
-SetSMUserThemePath(char *path)
+SetSMProgName(const char *name)
 {
-   userthemepath = duplicate(path);
+   command = Estrdup(name);
+}
+
+void
+SetSMUserThemePath(const char *path)
+{
+   userthemepath = Estrdup(path);
 }
 
 /* Used by multiheaded child processes to identify when they have
@@ -165,14 +185,14 @@ SetSMUserThemePath(char *path)
 static int          stale_sm_file = 0;
 
 void
-SetSMFile(char *path)
+SetSMFile(const char *path)
 {
    if (sm_file)
       Efree(sm_file);
    if (!path)
-      sm_file = duplicate(default_save_prefix());
+      sm_file = Estrdup(default_save_prefix());
    else
-      sm_file = duplicate(path);
+      sm_file = Estrdup(path);
    stale_sm_file = 0;
 }
 
@@ -188,66 +208,64 @@ GetGenericSMFile(void)
    return default_save_prefix();
 }
 
-/* This code covers X11R6 and X11R5 clients (xterm, KDE, ...). */
 static void
 SaveWindowStates(void)
 {
-   EWin              **lst, *ewin;
+   EWin               *const *lst, *ewin;
    int                 i, num, x, y;
    FILE               *f;
    char                s[4096], ss[4096];
 
-   lst = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
-   if (lst)
+   if (!Mode.wm.save_ok)
+      return;
+
+   Etmp(s);
+   f = fopen(s, "w");
+   if (f == NULL)
+      return;
+
+   lst = EwinListGetAll(&num);
+   for (i = 0; i < num; i++)
      {
-	Etmp(s);
-	f = fopen(s, "w");
-	if (f)
+	ewin = lst[i];
+	if ((!(ewin->internal))
+	    && ((ewin->icccm.wm_command) || (ewin->session_id)))
 	  {
-	     for (i = 0; i < num; i++)
+	     x = 0;
+	     y = 0;
+	     if (!ewin->sticky)
 	       {
-		  ewin = lst[i];
-		  if ((!(ewin->internal))
-		      && ((ewin->client.command) || (ewin->session_id)))
-		    {
-		       x = 0;
-		       y = 0;
-		       if (!ewin->sticky)
-			 {
-			    x = desks.desk[ewin->desktop].current_area_x *
-			       root.w;
-			    y = desks.desk[ewin->desktop].current_area_y *
-			       root.h;
-			 }
-		       fprintf(f, "[CLIENT] %i %i %i %i %i %i %i %i %i\n",
-			       ewin->x + x, ewin->y + y, ewin->client.w,
-			       ewin->client.h, ewin->desktop, ewin->iconified,
-			       ewin->shaded, ewin->sticky, ewin->layer);
-		       if (ewin->session_id)
-			  fprintf(f, "  [SESSION_ID] %s\n", ewin->session_id);
-		       if (ewin->client.name)
-			  fprintf(f, "  [NAME] %s\n", ewin->client.name);
-		       if (ewin->client.class)
-			  fprintf(f, "  [CLASS] %s\n", ewin->client.class);
-		       if (ewin->client.role)
-			  fprintf(f, "  [ROLE] %s\n", ewin->client.role);
-		       if (ewin->client.command)
-			  fprintf(f, "  [COMMAND] %s\n", ewin->client.command);
-		    }
+		  x = desks.desk[ewin->desktop].current_area_x * VRoot.w;
+		  y = desks.desk[ewin->desktop].current_area_y * VRoot.h;
 	       }
-	     fclose(f);
-	     Esnprintf(ss, sizeof(ss), "%s.clients.%i", GetSMFile(), root.scr);
-	     mv(s, ss);
-	     if (!isfile(ss))
-		Alert(_
-		      ("There was an error writing the clients "
-		       "session save file.\n" "You may have run out of disk "
-		       "space, not have permission\n"
-		       "to write to your filing system "
-		       "or other similar problems.\n"));
+	     fprintf(f, "[CLIENT] %i %i %i %i %i %i %i %i %i\n",
+		     ewin->x + x, ewin->y + y, ewin->client.w,
+		     ewin->client.h, ewin->desktop, ewin->iconified,
+		     ewin->shaded, ewin->sticky, ewin->layer);
+	     if (ewin->session_id)
+		fprintf(f, "  [SESSION_ID] %s\n", ewin->session_id);
+	     if (ewin->icccm.wm_res_name)
+		fprintf(f, "  [NAME] %s\n", ewin->icccm.wm_res_name);
+	     if (ewin->icccm.wm_res_class)
+		fprintf(f, "  [CLASS] %s\n", ewin->icccm.wm_res_class);
+	     if (ewin->icccm.wm_role)
+		fprintf(f, "  [ROLE] %s\n", ewin->icccm.wm_role);
+	     if (ewin->icccm.wm_command)
+		fprintf(f, "  [COMMAND] %s\n", ewin->icccm.wm_command);
 	  }
-	Efree(lst);
      }
+   fclose(f);
+
+   Esnprintf(ss, sizeof(ss), "%s.clients.%i", GetSMFile(), VRoot.scr);
+   if (EventDebug(EDBUG_TYPE_SESSION))
+      Eprintf("SaveWindowStates: %s\n", ss);
+   E_mv(s, ss);
+   if (!isfile(ss))
+      Alert(_("There was an error writing the clients "
+	      "session save file.\n" "You may have run out of disk "
+	      "space, not have permission\n"
+	      "to write to your filing system "
+	      "or other similar problems.\n"));
 }
 
 static void
@@ -256,12 +274,8 @@ LoadWindowStates(void)
    FILE               *f;
    char                s[4096], s1[4096];
 
-   Esnprintf(s, sizeof(s), "%s.clients.%i", GetSMFile(), root.scr);
-#ifndef __EMX__
+   Esnprintf(s, sizeof(s), "%s.clients.%i", GetSMFile(), VRoot.scr);
    f = fopen(s, "r");
-#else
-   f = fopen(s, "rt");
-#endif
    if (f)
      {
 	while (fgets(s, sizeof(s), f))
@@ -291,24 +305,24 @@ LoadWindowStates(void)
 	       }
 	     else if (!strcmp(s1, "[SESSION_ID]"))
 	       {
-		  matches[num_match - 1].session_id = duplicate(atword(s, 2));
+		  matches[num_match - 1].session_id = Estrdup(atword(s, 2));
 	       }
 	     else if (!strcmp(s1, "[NAME]"))
 	       {
-		  matches[num_match - 1].name = duplicate(atword(s, 2));
+		  matches[num_match - 1].name = Estrdup(atword(s, 2));
 	       }
 	     else if (!strcmp(s1, "[CLASS]"))
 	       {
-		  matches[num_match - 1].class = duplicate(atword(s, 2));
+		  matches[num_match - 1].class = Estrdup(atword(s, 2));
 	       }
 	     else if (!strcmp(s1, "[ROLE]"))
 	       {
-		  matches[num_match - 1].role = duplicate(atword(s, 2));
+		  matches[num_match - 1].role = Estrdup(atword(s, 2));
 		  /* Needed for matching X11R5 clients */
 	       }
 	     else if (!strcmp(s1, "[COMMAND]"))
 	       {
-		  matches[num_match - 1].command = duplicate(atword(s, 2));
+		  matches[num_match - 1].command = Estrdup(atword(s, 2));
 	       }
 	  }
 	fclose(f);
@@ -355,12 +369,12 @@ MatchEwinToSM(EWin * ewin)
 		   * FIXME: Mozilla DELETES the WM_COMMAND property on 
 		   * a regular basis so is is wise NOT to update 
 		   * this property when it is set to NULL. */
-		  if ((ewin->client.command) && (matches[i].command)
-		      && strcmp(ewin->client.command, matches[i].command))
+		  if ((ewin->icccm.wm_command) && (matches[i].command)
+		      && strcmp(ewin->icccm.wm_command, matches[i].command))
 		     continue;
 	       }
 
-	     if ((ewin->client.role) && (matches[i].role))
+	     if ((ewin->icccm.wm_role) && (matches[i].role))
 	       {
 		  /* The X11R6 protocol guarantees that any WM_WINDOW_ROLE
 		   * is unique among the windows sharing a SM_CLIENT_ID.
@@ -368,17 +382,17 @@ MatchEwinToSM(EWin * ewin)
 		   * Clients which use the same WM_WINDOW_ROLE on two 
 		   * windows are breaking the ICCCM even if they have 
 		   * different WM_CLASS properties on those windows. */
-		  if (strcmp(ewin->client.role, matches[i].role))
+		  if (strcmp(ewin->icccm.wm_role, matches[i].role))
 		     continue;
 	       }
 	     else
 	       {
 		  /* The WM_CLASS is a stable basis for a test. */
-		  if ((ewin->client.class) && (matches[i].class)
-		      && (strcmp(ewin->client.class, matches[i].class)))
+		  if ((ewin->icccm.wm_res_class) && (matches[i].class)
+		      && (strcmp(ewin->icccm.wm_res_class, matches[i].class)))
 		     continue;
-		  if ((ewin->client.name) && (matches[i].name)
-		      && (strcmp(ewin->client.name, matches[i].name)))
+		  if ((ewin->icccm.wm_res_name) && (matches[i].name)
+		      && (strcmp(ewin->icccm.wm_res_name, matches[i].name)))
 		     continue;
 
 		  /* Twm also matches on the WM_NAME but only when this value
@@ -388,7 +402,7 @@ MatchEwinToSM(EWin * ewin)
 
 	     matches[i].used = 1;
 	     ewin->client.already_placed = 1;
-	     ewin->iconified = matches[i].iconified;
+	     ewin->client.start_iconified = matches[i].iconified;
 	     ewin->sticky = matches[i].sticky;
 	     ewin->shaded = matches[i].shaded;
 	     ewin->layer = matches[i].layer;
@@ -398,15 +412,20 @@ MatchEwinToSM(EWin * ewin)
 	     if (!((ewin->client.already_placed) && (ewin->session_id)))
 	       {
 		  ewin->client.x = matches[i].x -
-		     (desks.desk[ewin->desktop].current_area_x * root.w);
+		     (desks.desk[ewin->desktop].current_area_x * VRoot.w);
 		  ewin->client.y = matches[i].y -
-		     (desks.desk[ewin->desktop].current_area_y * root.h);
+		     (desks.desk[ewin->desktop].current_area_y * VRoot.h);
+		  ewin->client.grav = NorthWestGravity;
 		  ewin->client.w = matches[i].w;
 		  ewin->client.h = matches[i].h;
 		  EMoveResizeWindow(disp, ewin->client.win, ewin->client.x,
 				    ewin->client.y, ewin->client.w,
 				    ewin->client.h);
 	       }
+	     if (EventDebug(EDBUG_TYPE_SNAPS))
+		Eprintf("Snap get sess  %#lx: %4d+%4d %4dx%4d: %s\n",
+			ewin->client.win, ewin->client.x, ewin->client.y,
+			ewin->client.w, ewin->client.h, EwinGetTitle(ewin));
 	     break;
 	  }
      }
@@ -416,29 +435,25 @@ MatchEwinToSM(EWin * ewin)
 void
 autosave(void)
 {
-   if (mode.startup)
+   if (!Mode.wm.save_ok)
       return;
-   if (mode.autosave)
+
+   if (Conf.autosave)
      {
 	char                s[4096];
 
 	Real_SaveSnapInfo(0, NULL);
+
 	Etmp(s);
 	SaveUserControlConfig(fopen(s, "w"));
-	mv(s, GetGenericSMFile());
+	if (EventDebug(EDBUG_TYPE_SESSION))
+	   Eprintf("autosave: save %s\n", GetGenericSMFile());
+	E_mv(s, GetGenericSMFile());
 	if (!isfile(GetGenericSMFile()))
-	   Alert(_
-		 ("There was an error saving your autosave data - filing\n"
-		  "system problems.\n"));
-/*      
- * if (strcmp(GetSMFile(), GetGenericSMFile()))
- * {
- * if (exists(GetGenericSMFile()))
- * rm(GetGenericSMFile());
- * symlink(GetSMFile(), GetGenericSMFile());
- * }
- */
+	   Alert(_("There was an error saving your autosave data - filing\n"
+		   "system problems.\n"));
      }
+#if 0				/* Why nuke it? */
    else
      {
 /*      char                buf[1024];
@@ -446,201 +461,14 @@ autosave(void)
  *      Esnprintf(buf, sizeof(buf) / sizeof(char), "rm %s*", GetSMFile());
  *      system(buf); */
 
-	rm(GetGenericSMFile());
+	if (EventDebug(EDBUG_TYPE_SESSION))
+	   Eprintf("autosave: kill %s\n", GetGenericSMFile());
+	E_rm(GetGenericSMFile());
      }
-}
-
-#ifndef HAVE_X11_SM_SMLIB_H
-
-void
-SessionInit(void)
-{
-   int                 i;
-
-   LoadWindowStates();
-}
-
-void
-ProcessICEMSGS(void)
-{
-}
-
-int
-GetSMfd(void)
-{
-   return -1;
-}
-
-void
-SessionGetInfo(EWin * ewin, Atom atom_change)
-{
-}
-
-void
-SetSMID(char *smid)
-{
-}
-
-void
-SaveSession(int shutdown)
-{
-   /* don't need */
-   /* autosave(); */
-}
-
-static void
-LogoutCB(int val, void *data)
-{
-   doSMExit(NULL);
-   val = 0;
-   data = NULL;
-}
-
-static void
-CB_SettingsEscape(int val, void *data)
-{
-   DialogClose((Dialog *) data);
-   val = 0;
-}
-
-/* This is the original code from actions.c(doExit). */
-void
-doSMExit(void *params)
-{
-   char                s[1024];
-   char               *real_exec;
-   char                sss[FILEPATH_LEN_MAX];
-   Window              w;
-   char                master_flag, do_master_kill;
-
-   master_flag = (master_pid == getpid())? 1 : 0;
-
-   do_master_kill = 1;
-
-   if (!params)
-      SaveSession(1);
-   if ((disp) && ((!params) || ((params) && strcmp((char *)params, "logout"))))
-      SetEInfoOnAll();
-
-   if (params)
-     {
-	SoundExit();
-#ifndef __EMX__
-	setsid();
 #endif
-	sscanf(params, "%1000s", s);
-	if (mustdel)
-	  {
-#ifndef __EMX__
-	     Esnprintf(sss, sizeof(sss), "/bin/rm -rf %s", themepath);
-#else
-	     Esnprintf(sss, sizeof(sss), "rm.exe -rf %s", themepath);
-#endif
-	     system(sss);
-	  }
-	if (!strcmp(s, "restart"))
-	  {
-	     AUDIO_PLAY("SOUND_WAIT");
-	     if (sound_fd >= 0)
-		close(sound_fd);
-	     w = MakeExtInitWin();
-	     XCloseDisplay(disp);
-	     disp = NULL;
-
-	     if (themepath[0] != 0)
-	       {
-		  Esnprintf(sss, sizeof(sss),
-			    "exec %s -single -ext_init_win %i -theme %s "
-			    "-econfdir %s -ecachedir %s -display %s", command,
-			    w, themename, UserEDir(), UserCacheDir(), dstr);
-		  execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", sss, NULL);
-	       }
-	     else
-	       {
-		  Esnprintf(sss, sizeof(sss),
-			    "exec %s -single -ext_init_win %i "
-			    "-econfdir %s -ecachedir %s -display %s", command,
-			    w, UserEDir(), UserCacheDir(), dstr);
-		  execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", sss, NULL);
-	       }
-	  }
-	else if (!strcmp(s, "restart_theme"))
-	  {
-	     AUDIO_PLAY("SOUND_WAIT");
-	     if (sound_fd >= 0)
-		close(sound_fd);
-	     w = MakeExtInitWin();
-	     XCloseDisplay(disp);
-	     disp = NULL;
-	     sscanf(params, "%*s %1000s", s);
-	     Esnprintf(sss, sizeof(sss),
-		       "exec %s -single -ext_init_win %i -theme %s "
-		       "-econfdir %s -ecachedir %s -display %s", command, w, s,
-		       UserEDir(), UserCacheDir(), dstr);
-	     execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", sss, NULL);
-	  }
-	else if (!strcmp(s, "restart_wm"))
-	  {
-#if ENABLE_KDE
-	     /* kill off kde */
-	     if (mode.kde_support)
-		KDE_Shutdown();
-#endif
-	     AUDIO_PLAY("SOUND_EXIT");
-	     if (sound_fd >= 0)
-		close(sound_fd);
-	     XCloseDisplay(disp);
-	     disp = NULL;
-	     if (atword(params, 2))
-		strncpy(s, atword(params, 2), 1000);
-	     real_exec = (char *)Emalloc(strlen(s) + 6);
-	     sprintf(real_exec,
-		     "exec %s " "-econfdir %s -ecachedir %s -display %s", s,
-		     UserEDir(), UserCacheDir(), dstr);
-	     execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", "exec", real_exec,
-		   NULL);
-	  }
-	else if (!strcmp(s, "logout"))
-	  {
-	     Dialog             *d;
-	     EWin               *ewin;
-
-	     if (!
-		 (d =
-		  FindItem("LOGOUT_DIALOG", 0, LIST_FINDBY_NAME,
-			   LIST_TYPE_DIALOG)))
-	       {
-		  AUDIO_PLAY("SOUND_LOGOUT");
-		  d = CreateDialog("LOGOUT_DIALOG");
-		  DialogSetTitle(d, "Are you sure?");
-		  DialogSetText(d,
-				"\n" "\n"
-				"    Are you sure you wish to log out ?    \n"
-				"\n" "\n");
-		  DialogAddButton(d, "  Yes, Log Out  ", LogoutCB, 1);
-		  DialogAddButton(d, "  No  ", NULL, 1);
-		  DialogBindKey(d, "Escape", CB_SettingsEscape, 0, d);
-		  DialogBindKey(d, "Return", LogoutCB, 0, d);
-	       }
-	     ShowDialog(d);
-	     ewin = FindEwinByDialog(d);
-	     if (ewin)
-	       {
-		  ArrangeEwinCentered(ewin, 1);
-	       }
-	     return;
-	  }
-     }
-#if ENABLE_KDE
-   /* kill off kde */
-   if (mode.kde_support)
-      KDE_Shutdown();
-#endif
-   AUDIO_PLAY("SOUND_EXIT");
-   EExit(0);
 }
 
-#else /* HAVE_X11_SM_SMLIB_H */
+#ifdef HAVE_X11_SM_SMLIB_H
 
 #include <X11/SM/SMlib.h>
 
@@ -651,7 +479,6 @@ doSMExit(void *params)
 
 static char        *sm_client_id = NULL;
 static SmcConn      sm_conn = NULL;
-static int          sm_fd = -1;
 
 /* True if we are saving state for a doExit("restart") */
 static int          restarting = False;
@@ -661,15 +488,15 @@ set_save_props(SmcConn smc_conn, int master_flag)
 {
    char               *user = NULL;
    char               *program = NULL;
-   char               *pristr = "_GSM_Priority";
-   char               *smid = "-smid";
-   char               *single = "-single";
-   char               *smfile = "-smfile";
-   char               *econfdir = "-econfdir";
+   const char         *pristr = "_GSM_Priority";
+   const char         *smid = "-smid";
+   const char         *single = "-single";
+   const char         *smfile = "-smfile";
+   const char         *econfdir = "-econfdir";
    char               *e_conf_dir;
-   char               *ecachedir = "-ecachedir";
+   const char         *ecachedir = "-ecachedir";
    char               *e_cache_dir;
-   char               *extinitwin = "-ext_init_win";
+   const char         *extinitwin = "-ext_init_win";
    char                buf[512];
    char                priority = 10;
    char                style;
@@ -751,8 +578,8 @@ set_save_props(SmcConn smc_conn, int master_flag)
       style = SmRestartNever;
 
    user = username(getuid());
-   e_conf_dir = UserEDir();
-   e_cache_dir = UserCacheDir();
+   e_conf_dir = EDirUser();
+   e_cache_dir = EDirUserCache();
    /* The SM specs state that the SmProgram should be the argument passed
     * to execve. Passing argv[0] is close enough. */
    program = command;
@@ -768,7 +595,7 @@ set_save_props(SmcConn smc_conn, int master_flag)
 
 #if USE_DISCARD_PROPERTY
    /* Tell session manager how to clean up our old data */
-   Esnprintf(buf, sizeof(buf) / sizeof(char), "rm %s*.clients.*", sm_file);
+   Esnprintf(buf, sizeof(buf), "rm %s*.clients.*", sm_file);
 
    discardVal[0].length = strlen(sh);
    discardVal[0].value = sh;
@@ -781,34 +608,34 @@ set_save_props(SmcConn smc_conn, int master_flag)
    n = 0;
    restartVal[n].length = strlen(command);
    restartVal[n++].value = command;
-   if (single_screen_mode)
+   if (Mode.wm.single)
      {
 	restartVal[n].length = strlen(single);
-	restartVal[n++].value = single;
+	restartVal[n++].value = (char *)single;
      }
    if (restarting)
      {
-	Esnprintf(buf, sizeof(buf) / sizeof(char), "%i", init_win_ext);
+	Esnprintf(buf, sizeof(buf), "%li", init_win_ext);
 
 	restartVal[n].length = strlen(extinitwin);
-	restartVal[n++].value = extinitwin;
+	restartVal[n++].value = (char *)extinitwin;
 	restartVal[n].length = strlen(buf);
 	restartVal[n++].value = buf;
      }
    restartVal[n].length = strlen(smfile);
-   restartVal[n++].value = smfile;
+   restartVal[n++].value = (char *)smfile;
    restartVal[n].length = strlen(sm_file);
    restartVal[n++].value = sm_file;
    restartVal[n].length = strlen(smid);
-   restartVal[n++].value = smid;
+   restartVal[n++].value = (char *)smid;
    restartVal[n].length = strlen(sm_client_id);
    restartVal[n++].value = sm_client_id;
    restartVal[n].length = strlen(econfdir);
-   restartVal[n++].value = econfdir;
+   restartVal[n++].value = (char *)econfdir;
    restartVal[n].length = strlen(e_conf_dir);
    restartVal[n++].value = e_conf_dir;
    restartVal[n].length = strlen(ecachedir);
-   restartVal[n++].value = ecachedir;
+   restartVal[n++].value = (char *)ecachedir;
    restartVal[n].length = strlen(e_cache_dir);
    restartVal[n++].value = e_cache_dir;
 
@@ -841,11 +668,9 @@ set_save_props(SmcConn smc_conn, int master_flag)
 static void
 callback_save_yourself2(SmcConn smc_conn, SmPointer client_data)
 {
-   char                master_flag = (master_pid == getpid());
-
    /* dont need anymore */
    /* autosave(); */
-   if (!master_flag)
+   if (!Mode.wm.master)
      {
 	struct timeval      tv1, tv2;
 
@@ -864,7 +689,7 @@ callback_save_yourself2(SmcConn smc_conn, SmPointer client_data)
 	  }
      }
    stale_sm_file = 1;
-   set_save_props(smc_conn, master_flag);
+   set_save_props(smc_conn, Mode.wm.master);
    SmcSaveYourselfDone(smc_conn, True);
    if (restarting)
       EExit(0);
@@ -875,7 +700,7 @@ static void
 callback_save_yourself(SmcConn smc_conn, SmPointer client_data, int save_style,
 		       Bool shutdown, int interact_style, Bool fast)
 {
-   if (master_pid == getpid())
+   if (Mode.wm.master)
      {
 	char                s[4096];
 
@@ -896,7 +721,7 @@ callback_save_yourself(SmcConn smc_conn, SmPointer client_data, int save_style,
  * if (strcmp(GetSMFile(), GetGenericSMFile()))
  * {
  * if (exists(GetGenericSMFile()))
- * rm(GetGenericSMFile());
+ * E_rm(GetGenericSMFile());
  * symlink(GetSMFile(), GetGenericSMFile());
  * }
  */
@@ -913,9 +738,8 @@ callback_save_yourself(SmcConn smc_conn, SmPointer client_data, int save_style,
 static void
 callback_die(SmcConn smc_conn, SmPointer client_data)
 {
-   if (master_pid == getpid())
-      ApplySclass(FindItem
-		  ("SOUND_EXIT", 0, LIST_FINDBY_NAME, LIST_TYPE_SCLASS));
+   if (Mode.wm.master)
+      SoundPlay("SOUND_EXIT");
    EExit(0);
    smc_conn = 0;
    client_data = NULL;
@@ -948,9 +772,12 @@ ice_io_error_handler(IceConn connection)
    connection = 0;
 }
 
+#endif /* HAVE_X11_SM_SMLIB_H */
+
 void
 SessionInit(void)
 {
+#ifdef HAVE_X11_SM_SMLIB_H
    static SmPointer    context;
    SmcCallbacks        callbacks;
 
@@ -973,7 +800,7 @@ SessionInit(void)
 	char               *client_id = NULL;
 
 	if (sm_client_id)
-	   client_id = duplicate(sm_client_id);
+	   client_id = Estrdup(sm_client_id);
 	sm_conn =
 	   SmcOpenConnection(NULL, &context, SmProtoMajor, SmProtoMinor,
 			     SmcSaveYourselfProcMask | SmcDieProcMask |
@@ -1000,8 +827,8 @@ SessionInit(void)
 	styleVal.length = 1;
 	styleVal.value = style;
 
-	styleProp.name = SmRestartStyleHint;
-	styleProp.type = SmCARD8;
+	styleProp.name = (char *)SmRestartStyleHint;
+	styleProp.type = (char *)SmCARD8;
 	styleProp.num_vals = 1;
 	styleProp.vals = &styleVal;
 
@@ -1014,12 +841,15 @@ SessionInit(void)
 	fcntl(sm_fd, F_SETFD, fcntl(sm_fd, F_GETFD, 0) | FD_CLOEXEC);
      }
    stale_sm_file = 1;
+#endif /* HAVE_X11_SM_SMLIB_H */
+
    LoadWindowStates();
 }
 
 void
 ProcessICEMSGS(void)
 {
+#ifdef HAVE_X11_SM_SMLIB_H
    IceProcessMessagesStatus status;
 
    if (sm_fd < 0)
@@ -1028,17 +858,17 @@ ProcessICEMSGS(void)
    if (status == IceProcessMessagesIOError)
      {
 	/* Less of the hope.... E survives */
-	DialogAlert(_
-		    ("ERROR!\n" "\n"
-		     "Lost the Session Manager that was there?\n"
-		     "Here here session manager... come here... want a bone?\n"
-		     "Oh come now! Stop sulking! Bugger. Oh well. "
-		     "Will continue without\n" "a session manager.\n" "\n"
-		     "I'll survive somehow.\n" "\n" "\n" "... I hope.\n"));
+	DialogAlert(_("ERROR!\n" "\n"
+		      "Lost the Session Manager that was there?\n"
+		      "Here here session manager... come here... want a bone?\n"
+		      "Oh come now! Stop sulking! Bugger. Oh well. "
+		      "Will continue without\n" "a session manager.\n" "\n"
+		      "I'll survive somehow.\n" "\n" "\n" "... I hope.\n"));
 	SmcCloseConnection(sm_conn, 0, NULL);
 	sm_conn = NULL;
 	sm_fd = -1;
      }
+#endif /* HAVE_X11_SM_SMLIB_H */
 }
 
 int
@@ -1050,6 +880,7 @@ GetSMfd(void)
 void
 SessionGetInfo(EWin * ewin, Atom atom_change)
 {
+#ifdef HAVE_X11_SM_SMLIB_H
    char               *s;
    Window             *w;
    int                 size;
@@ -1080,41 +911,49 @@ SessionGetInfo(EWin * ewin, Atom atom_change)
 	  }
 	Efree(w);
      }
+#endif /* HAVE_X11_SM_SMLIB_H */
 }
 
 void
-SetSMID(char *smid)
+SetSMID(const char *smid)
 {
-   sm_client_id = smid;
+#ifdef HAVE_X11_SM_SMLIB_H
+   sm_client_id = Estrdup(smid);
+#endif /* HAVE_X11_SM_SMLIB_H */
 }
 
 static void
-LogoutCB(int val, void *data)
+LogoutCB(int val __UNUSED__, void *data __UNUSED__)
 {
+#ifdef HAVE_X11_SM_SMLIB_H
    if (sm_conn)
      {
 	SmcRequestSaveYourself(sm_conn, SmSaveBoth, True, SmInteractStyleAny,
 			       False, True);
      }
    else
+#endif /* HAVE_X11_SM_SMLIB_H */
      {
-	doSMExit(NULL);
+	SessionExit(NULL);
      }
-   val = 0;
-   data = NULL;
 }
 
 void
 SaveSession(int shutdown)
 {
+   if (EventDebug(EDBUG_TYPE_SESSION))
+      Eprintf("SaveSession(%d)\n", shutdown);
+
    /* dont' need anymore */
-   /* ayosave(); */
+   /* autosave(); */
+#ifdef HAVE_X11_SM_SMLIB_H
    if (shutdown && sm_conn)
      {
 	SmcCloseConnection(sm_conn, 0, NULL);
 	sm_conn = NULL;
 	sm_fd = -1;
      }
+#endif /* HAVE_X11_SM_SMLIB_H */
 }
 
 static void
@@ -1124,6 +963,8 @@ CB_SettingsEscape(int val, void *data)
    val = 0;
 }
 
+#ifdef HAVE_X11_SM_SMLIB_H
+
 /*
  * Normally, the SM will throw away all the session data for a client
  * that breaks its connection unexpectedly. In order to avoid this we 
@@ -1133,14 +974,14 @@ CB_SettingsEscape(int val, void *data)
  * and then restore it on restart. We grab X input via the ext_init_win
  * so the our clients remain frozen while we are down.
  */
-void
-doSMExit(void *params)
+static void
+doSMExit(const void *params)
 {
    char                s[1024];
-   char                master_flag, do_master_kill;
+   Window              win = None;
 
-   master_flag = (master_pid == getpid())? 1 : 0;
-   do_master_kill = 1;
+   if (EventDebug(EDBUG_TYPE_SESSION))
+      Eprintf("doSMExit: %p\n", params);
 
    restarting = True;
 
@@ -1151,59 +992,36 @@ doSMExit(void *params)
    SaveWindowStates();
    if (!params)
       SaveSession(1);
-   if ((disp) && ((!params) || ((params) && strcmp((char *)params, "logout"))))
-      SetEInfoOnAll();
+   ICCCM_SetEInfoOnAll();
+   EwinsSetFree();
+   if (Mode.wm.startup && Mode.wm.exiting)
+      MapUnmap(1);
+
+   if (disp)
+     {
+	XSelectInput(disp, VRoot.win, 0);
+
+	if (Mode.wm.master && init_win_ext)
+	   XKillClient(disp, init_win_ext);
+     }
+
    if ((!params) || (!strcmp((char *)s, "exit")))
      {
 	callback_die(sm_conn, NULL);
      }
-   else if (!strcmp(s, "logout"))
-     {
-	Dialog             *d;
-	EWin               *ewin;
-
-	if (!
-	    (d =
-	     FindItem("LOGOUT_DIALOG", 0, LIST_FINDBY_NAME, LIST_TYPE_DIALOG)))
-	  {
-	     AUDIO_PLAY("SOUND_LOGOUT");
-	     d = CreateDialog("LOGOUT_DIALOG");
-	     DialogSetTitle(d, _("Are you sure?"));
-	     DialogSetText(d,
-			   _("\n" "\n"
-			     "    Are you sure you wish to log out ?    \n" "\n"
-			     "\n"));
-	     DialogAddButton(d, _("  Yes, Log Out  "), LogoutCB, 1);
-	     DialogAddButton(d, _("  No  "), NULL, 1);
-	     DialogBindKey(d, "Escape", CB_SettingsEscape, 1, d);
-	     DialogBindKey(d, "Return", LogoutCB, 0, d);
-	  }
-	ShowDialog(d);
-	ewin = FindEwinByDialog(d);
-	if (ewin)
-	  {
-	     ArrangeEwinCentered(ewin, 1);
-	  }
-	return;
-     }
    else if (!strcmp(s, "restart_wm"))
      {
-	AUDIO_PLAY("SOUND_WAIT");
-#if ENABLE_KDE
-	/* kill off kde */
-	if (mode.kde_support)
-	   KDE_Shutdown();
-#endif
+	SoundPlay("SOUND_WAIT");
 	XCloseDisplay(disp);
 	disp = NULL;
-	Esnprintf(s, sizeof(s), "exec %s -display %s", atword(params, 2), dstr);
+	Esnprintf(s, sizeof(s), "exec %s", atword(params, 2));
 	execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", s, NULL);
      }
    else if (!strcmp(s, "restart"))
      {
-	AUDIO_PLAY("SOUND_WAIT");
+	SoundPlay("SOUND_WAIT");
 	if (disp)
-	   init_win_ext = MakeExtInitWin();
+	   win = MakeExtInitWin();
 
 	if (disp)
 	  {
@@ -1214,41 +1032,42 @@ doSMExit(void *params)
 	  {
 	     if (sm_client_id)
 		Esnprintf(s, sizeof(s),
-			  "exec %s -single -ext_init_win %i -theme %s "
+			  "exec %s -single -ext_init_win %li -theme %s "
 			  "-econfdir %s -ecachedir %s "
-			  "-smfile %s -smid %s -display %s", command,
-			  init_win_ext, themename, UserEDir(), UserCacheDir(),
-			  sm_file, sm_client_id, dstr);
+			  "-smfile %s -smid %s", command,
+			  win, Conf.theme.name, EDirUser(),
+			  EDirUserCache(), sm_file, sm_client_id);
 	     else
 		Esnprintf(s, sizeof(s),
-			  "exec %s -single -ext_init_win %i -theme %s "
+			  "exec %s -single -ext_init_win %li -theme %s "
 			  "-econfdir %s -ecachedir %s "
-			  "-smfile %s -display %s", command, init_win_ext,
-			  themename, UserEDir(), UserCacheDir(), sm_file, dstr);
+			  "-smfile %s", command, win,
+			  Conf.theme.name, EDirUser(), EDirUserCache(),
+			  sm_file);
 	     execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", s, NULL);
 	  }
 	else
 	  {
 	     if (sm_client_id)
 		Esnprintf(s, sizeof(s),
-			  "exec %s -single -ext_init_win %i "
+			  "exec %s -single -ext_init_win %li "
 			  "-econfdir %s -ecachedir %s "
-			  "-smfile %s -smid %s -display %s", command,
-			  init_win_ext, UserEDir(), UserCacheDir(), sm_file,
-			  sm_client_id, dstr);
+			  "-smfile %s -smid %s", command,
+			  win, EDirUser(), EDirUserCache(),
+			  sm_file, sm_client_id);
 	     else
 		Esnprintf(s, sizeof(s),
-			  "exec %s -single -ext_init_win %i"
+			  "exec %s -single -ext_init_win %li"
 			  "-econfdir %s -ecachedir %s "
-			  "-smfile %s -display %s", command, init_win_ext,
-			  UserEDir(), UserCacheDir(), sm_file, dstr);
+			  "-smfile %s", command, win,
+			  EDirUser(), EDirUserCache(), sm_file);
 	     execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", s, NULL);
 	  }
      }
    else if (!strcmp((char *)s, "restart_theme"))
      {
-	AUDIO_PLAY("SOUND_WAIT");
-	init_win_ext = MakeExtInitWin();
+	SoundPlay("SOUND_WAIT");
+	win = MakeExtInitWin();
 	if (atword(params, 1) && strlen((char *)params) < 1024)
 	  {
 	     sscanf(params, "%*s %1000s", s);
@@ -1261,31 +1080,207 @@ doSMExit(void *params)
 	  }
 	if (sm_client_id)
 	   Esnprintf(s, sizeof(s),
-		     "exec %s -single -ext_init_win %i -theme %s "
+		     "exec %s -single -ext_init_win %li -theme %s "
 		     "-econfdir %s -ecachedir %s "
-		     "-smfile %s -smid %s -display %s", command, init_win_ext,
-		     userthemepath, UserEDir(), UserCacheDir(), sm_file,
-		     sm_client_id, dstr);
+		     "-smfile %s -smid %s", command, win,
+		     userthemepath, EDirUser(), EDirUserCache(), sm_file,
+		     sm_client_id);
 	else
 	   Esnprintf(s, sizeof(s),
-		     "exec %s -ext_init_win %i -theme %s "
+		     "exec %s -ext_init_win %li -theme %s "
 		     "-econfdir %s -ecachedir %s "
-		     "-smfile %s -single -display %s", command, init_win_ext,
-		     userthemepath, UserEDir(), UserCacheDir(), sm_file, dstr);
+		     "-smfile %s -single", command, win,
+		     userthemepath, EDirUser(), EDirUserCache(), sm_file);
 	execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", s, NULL);
      }
    else if (!strcmp((char *)s, "error"))
-      EExit(0);
+     {
+	EExit(0);
+     }
 
-#if ENABLE_KDE
-   /* kill off kde */
-   if (mode.kde_support)
-      KDE_Shutdown();
-#endif
    restarting = False;
    SaveSession(1);
-   AUDIO_PLAY("SOUND_EXIT");
+   SoundPlay("SOUND_EXIT");
+   EExit(0);
+}
+
+#else /* HAVE_X11_SM_SMLIB_H */
+
+/* This is the original code from actions.c(doExit). */
+static void
+doSMExit(const void *params)
+{
+   char                s[1024];
+   char               *real_exec;
+   char                sss[FILEPATH_LEN_MAX];
+   Window              win;
+
+   if (!params)
+      SaveSession(1);
+   ICCCM_SetEInfoOnAll();
+
+   if (params)
+     {
+	SoundExit();
+	setsid();
+	sscanf(params, "%1000s", s);
+	ThemeCleanup();
+
+	if (!strcmp(s, "restart"))
+	  {
+	     SoundPlay("SOUND_WAIT");
+	     win = MakeExtInitWin();
+	     XCloseDisplay(disp);
+	     disp = NULL;
+
+	     if (themepath[0] != 0)
+	       {
+		  Esnprintf(sss, sizeof(sss),
+			    "exec %s -single -ext_init_win %li -theme %s "
+			    "-econfdir %s -ecachedir %s", command,
+			    win, Conf.theme.name, EDirUser(), EDirUserCache());
+		  execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", sss, NULL);
+	       }
+	     else
+	       {
+		  Esnprintf(sss, sizeof(sss),
+			    "exec %s -single -ext_init_win %li "
+			    "-econfdir %s -ecachedir %s", command,
+			    win, EDirUser(), EDirUserCache());
+		  execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", sss, NULL);
+	       }
+	  }
+	else if (!strcmp(s, "restart_theme"))
+	  {
+	     SoundPlay("SOUND_WAIT");
+	     win = MakeExtInitWin();
+	     XCloseDisplay(disp);
+	     disp = NULL;
+	     sscanf(params, "%*s %1000s", s);
+	     Esnprintf(sss, sizeof(sss),
+		       "exec %s -single -ext_init_win %li -theme %s "
+		       "-econfdir %s -ecachedir %s", command, win, s,
+		       EDirUser(), EDirUserCache());
+	     execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", sss, NULL);
+	  }
+	else if (!strcmp(s, "restart_wm"))
+	  {
+	     SoundPlay("SOUND_EXIT");
+	     XCloseDisplay(disp);
+	     disp = NULL;
+	     if (atword(params, 2))
+		strncpy(s, atword(params, 2), 1000);
+	     real_exec = (char *)Emalloc(strlen(s) + 6);
+	     sprintf(real_exec,
+		     "exec %s " "-econfdir %s -ecachedir %s", s,
+		     EDirUser(), EDirUserCache());
+	     execl(DEFAULT_SH_PATH, DEFAULT_SH_PATH, "-c", "exec", real_exec,
+		   NULL);
+	  }
+     }
+
+   SoundPlay("SOUND_EXIT");
    EExit(0);
 }
 
 #endif /* HAVE_X11_SM_SMLIB_H */
+
+static void
+SessionLogoutConfirm(void)
+{
+   Dialog             *d;
+   EWin               *ewin;
+
+   d = FindItem("LOGOUT_DIALOG", 0, LIST_FINDBY_NAME, LIST_TYPE_DIALOG);
+   if (!d)
+     {
+	SoundPlay("SOUND_LOGOUT");
+	d = DialogCreate("LOGOUT_DIALOG");
+	DialogSetTitle(d, _("Are you sure?"));
+	DialogSetText(d, _("\n\n"
+			   "    Are you sure you wish to log out ?    \n"
+			   "\n\n"));
+	DialogAddButton(d, _("  Yes, Log Out  "), LogoutCB, 1);
+	DialogAddButton(d, _("  No  "), NULL, 1);
+	DialogBindKey(d, "Escape", CB_SettingsEscape, 1, d);
+	DialogBindKey(d, "Return", LogoutCB, 0, d);
+     }
+
+   ShowDialog(d);
+
+   ewin = FindEwinByDialog(d);
+   if (ewin)
+      ArrangeEwinCentered(ewin, 1);
+   return;
+}
+
+int
+SessionExit(const void *param)
+{
+   if (param && !strcmp(param, "logout"))
+     {
+	SessionLogoutConfirm();
+	return 0;
+     }
+
+   if (Mode.wm.exiting++)
+     {
+	/* This may be possible during nested signal handling */
+	Eprintf("SessionExit already in progress ... now exiting\n");
+	exit(1);
+     }
+
+   doSMExit(param);
+   return 0;
+}
+
+int
+EExit(int exitcode)
+{
+   int                 i;
+
+   EDBUG(9, "EExit");
+
+   if (EventDebug(EDBUG_TYPE_SESSION))
+      Eprintf("EExit(%d)\n", exitcode);
+
+   SaveSession(1);
+
+   if (disp)
+     {
+	UngrabX();
+	GrabPointerRelease();
+	XAllowEvents(disp, AsyncBoth, CurrentTime);
+
+	/* This mechanism is only needed when the SM is unavailable: */
+	ICCCM_SetEInfoOnAll();
+
+	/* XSetInputFocus(disp, None, RevertToParent, CurrentTime); */
+	/* I think this is a better way to release the grabs: (felix) */
+	XSetInputFocus(disp, PointerRoot, RevertToPointerRoot, CurrentTime);
+	XSelectInput(disp, VRoot.win, 0);
+	XCloseDisplay(disp);
+     }
+
+   XSetErrorHandler((XErrorHandler) NULL);
+   XSetIOErrorHandler((XIOErrorHandler) NULL);
+
+   SignalsRestore();
+
+   if (Mode.wm.master)
+     {
+	SoundExit();
+	ThemeCleanup();
+	for (i = 0; i < child_count; i++)
+	   kill(e_children[i], SIGINT);
+     }
+   else
+     {
+	exitcode = 0;
+     }
+
+   Real_SaveSnapInfo(0, NULL);
+
+   exit(exitcode);
+   EDBUG_RETURN(exitcode);
+}
