@@ -1,6 +1,5 @@
-
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -22,9 +21,27 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
-#ifdef __EMX__
-#include <process.h>
-#endif
+
+static Snapshot    *NewSnapshot(const char *name);
+
+/* Format the window identifier string */
+static int
+EwinMakeID(EWin * ewin, char *buf, int len)
+{
+   if ((ewin->icccm.wm_role) && (ewin->icccm.wm_res_name)
+       && (ewin->icccm.wm_res_class))
+      Esnprintf(buf, len, "%s.%s:%s", ewin->icccm.wm_res_name,
+		ewin->icccm.wm_res_class, ewin->icccm.wm_role);
+   else if ((ewin->icccm.wm_res_name) && (ewin->icccm.wm_res_class))
+      Esnprintf(buf, len, "%s.%s", ewin->icccm.wm_res_name,
+		ewin->icccm.wm_res_class);
+   else if (ewin->icccm.wm_name)
+      Esnprintf(buf, len, "TITLE.%s", ewin->icccm.wm_name);
+   else
+      return -1;
+
+   return 0;
+}
 
 /* find a snapshot state that applies to this ewin */
 Snapshot           *
@@ -35,25 +52,23 @@ FindSnapshot(EWin * ewin)
 
    if (ewin->snap)
       return ewin->snap;
-   if ((ewin->client.name) && (ewin->client.class))
-      Esnprintf(buf, sizeof(buf), "%s.%s", ewin->client.name,
-		ewin->client.class);
-   else if (ewin->client.title)
-      Esnprintf(buf, sizeof(buf), "TITLE.%s", ewin->client.title);
-   else
+
+   if (EwinMakeID(ewin, buf, sizeof(buf)))
       return NULL;
+
    sn = FindItem(buf, 0, LIST_FINDBY_BOTH, LIST_TYPE_SNAPSHOT);
    if (sn)
      {
 	ListChangeItemID(LIST_TYPE_SNAPSHOT, sn, 1);
 	sn->used = 1;
      }
+
    return sn;
 }
 
 /* find a snapshot state that applies to this ewin Or if that doesnt exist */
 /* create a new one */
-Snapshot           *
+static Snapshot    *
 GetSnapshot(EWin * ewin)
 {
    Snapshot           *sn;
@@ -63,39 +78,38 @@ GetSnapshot(EWin * ewin)
      {
 	char                buf[4096];
 
-	if ((ewin->client.name) && (ewin->client.class))
-	   Esnprintf(buf, sizeof(buf), "%s.%s", ewin->client.name,
-		     ewin->client.class);
-	else if (ewin->client.title)
-	   Esnprintf(buf, sizeof(buf), "TITLE.%s", ewin->client.title);
+	if (EwinMakeID(ewin, buf, sizeof(buf)))
+	   return NULL;
+
 	sn = NewSnapshot(buf);
 	ListChangeItemID(LIST_TYPE_SNAPSHOT, sn, 1);
-	if ((ewin->client.name) && (ewin->client.class))
+	if ((ewin->icccm.wm_res_name) && (ewin->icccm.wm_res_class))
 	  {
 	     sn->win_title = NULL;
-	     sn->win_name = duplicate(ewin->client.name);
-	     sn->win_class = duplicate(ewin->client.class);
+	     sn->win_name = Estrdup(ewin->icccm.wm_res_name);
+	     sn->win_class = Estrdup(ewin->icccm.wm_res_class);
 	  }
 	else
 	  {
-	     sn->win_title = duplicate(ewin->client.title);
+	     sn->win_title = Estrdup(ewin->icccm.wm_name);
 	     sn->win_name = NULL;
 	     sn->win_class = NULL;
 	  }
 	sn->used = 1;
 	ewin->snap = sn;
      }
+
    return sn;
 }
 
 /* create a new snapshot */
-Snapshot           *
-NewSnapshot(char *name)
+static Snapshot    *
+NewSnapshot(const char *name)
 {
    Snapshot           *sn;
 
    sn = Emalloc(sizeof(Snapshot));
-   sn->name = duplicate(name);
+   sn->name = Estrdup(name);
    sn->win_title = NULL;
    sn->win_name = NULL;
    sn->win_class = NULL;
@@ -129,9 +143,11 @@ NewSnapshot(char *name)
    sn->use_neverfocus = 0;
    sn->neverfocus = 0;
    AddItemEnd(sn, sn->name, 0, LIST_TYPE_SNAPSHOT);
+
    return sn;
 }
 
+#if 0				/* Not used */
 /* clear all information out of a snapshot and set its refernce use to 0 */
 void
 ClearSnapshot(Snapshot * sn)
@@ -174,6 +190,7 @@ ClearSnapshot(Snapshot * sn)
    sn->neverfocus = 0;
    ListChangeItemID(LIST_TYPE_SNAPSHOT, sn, 0);
 }
+#endif
 
 static void         CB_ApplySnapEscape(int val, void *data);
 static void
@@ -213,10 +230,12 @@ CB_ApplySnap(int val, void *data)
 
 	     UnsnapshotEwin(ewin);
 	     sn = GetSnapshot(ewin);
+#if 0				/* ?!? */
 	     if (sn)
 	       {
 		  ClearSnapshot(sn);
 	       }
+#endif
 	     if (tmp_snap_border)
 		SnapshotEwinBorder(ewin);
 	     if (tmp_snap_desktop)
@@ -260,13 +279,13 @@ SnapshotEwinDialog(EWin * ewin)
 	ShowDialog(d);
 	return;
      }
-   d = CreateDialog("SNAPSHOT_WINDOW");
+   d = DialogCreate("SNAPSHOT_WINDOW");
    DialogSetTitle(d, _("Remembered Application Attributes"));
 
    table = DialogInitItem(d);
    DialogItemTableSetOptions(table, 4, 0, 0, 0);
 
-   if (mode.dialog_headers)
+   if (Conf.dialogs.headers)
      {
 	di = DialogAddItem(table, DITEM_IMAGE);
 	DialogItemSetPadding(di, 2, 2, 2, 2);
@@ -343,9 +362,9 @@ SnapshotEwinDialog(EWin * ewin)
    DialogItemSetPadding(di, 2, 2, 2, 2);
    DialogItemSetFill(di, 1, 0);
    DialogItemSetAlign(di, 1024, 512);
-   DialogItemTextSetText(di, ewin->client.title);
+   DialogItemTextSetText(di, ewin->icccm.wm_name);
 
-   if (ewin->client.name)
+   if (ewin->icccm.wm_res_name)
      {
 	di = DialogAddItem(table, DITEM_TEXT);
 	DialogItemSetPadding(di, 2, 2, 2, 2);
@@ -358,10 +377,10 @@ SnapshotEwinDialog(EWin * ewin)
 	DialogItemSetPadding(di, 2, 2, 2, 2);
 	DialogItemSetFill(di, 1, 0);
 	DialogItemSetAlign(di, 1024, 512);
-	DialogItemTextSetText(di, ewin->client.name);
+	DialogItemTextSetText(di, ewin->icccm.wm_res_name);
      }
 
-   if (ewin->client.class)
+   if (ewin->icccm.wm_res_class)
      {
 	di = DialogAddItem(table, DITEM_TEXT);
 	DialogItemSetPadding(di, 2, 2, 2, 2);
@@ -374,10 +393,10 @@ SnapshotEwinDialog(EWin * ewin)
 	DialogItemSetPadding(di, 2, 2, 2, 2);
 	DialogItemSetFill(di, 1, 0);
 	DialogItemSetAlign(di, 1024, 512);
-	DialogItemTextSetText(di, ewin->client.class);
+	DialogItemTextSetText(di, ewin->icccm.wm_res_class);
      }
 
-   if (ewin->client.command)
+   if (ewin->icccm.wm_command)
      {
 	di = DialogAddItem(table, DITEM_TEXT);
 	DialogItemSetPadding(di, 2, 2, 2, 2);
@@ -392,23 +411,23 @@ SnapshotEwinDialog(EWin * ewin)
 	DialogItemSetAlign(di, 1024, 512);
 
 	/* if the command is long, cut in into slices of about 80 characters */
-	if (strlen(ewin->client.command) > 80)
+	if (strlen(ewin->icccm.wm_command) > 80)
 	  {
 	     int                 i = 0, slice, last;
 
 	     s[0] = 0;
 	     slice = 64;
-	     while ((i <= (int)strlen(ewin->client.command))
+	     while ((i <= (int)strlen(ewin->icccm.wm_command))
 		    && (i < (int)(sizeof(s) / 4)))
 	       {
 		  last = i;
 		  i += 64;
 		  slice = 64;
 		  /* and make sure that we don't cut in the middle of a word. */
-		  while ((ewin->client.command[i++] != ' ')
+		  while ((ewin->icccm.wm_command[i++] != ' ')
 			 && (i < (int)(sizeof(s) / 4)))
 		     slice++;
-		  strncat(s, ewin->client.command + last, slice);
+		  strncat(s, ewin->icccm.wm_command + last, slice);
 		  if (i < (int)(sizeof(s) / 4))
 		     strcat(s, "\n");
 		  else
@@ -417,7 +436,7 @@ SnapshotEwinDialog(EWin * ewin)
 	     DialogItemTextSetText(di, s);
 	  }
 	else
-	   DialogItemTextSetText(di, ewin->client.command);
+	   DialogItemTextSetText(di, ewin->icccm.wm_command);
      }
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
@@ -490,7 +509,7 @@ SnapshotEwinDialog(EWin * ewin)
    DialogItemCheckButtonSetState(di, tmp_snap_skiplists);
    DialogItemCheckButtonSetPtr(di, &tmp_snap_skiplists);
 
-#if 0
+#if 0				/* Disabled (why?) */
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetPadding(di, 2, 2, 2, 2);
@@ -499,13 +518,13 @@ SnapshotEwinDialog(EWin * ewin)
    DialogItemCheckButtonSetState(di, tmp_snap_neverfocus);
    DialogItemCheckButtonSetPtr(di, &tmp_snap_neverfocus);
 #endif
-   if (ewin->client.command)
+   if (ewin->icccm.wm_command)
      {
 	char                ok = 1;
 
-	if (ewin->client.machine)
+	if (ewin->icccm.wm_machine)
 	  {
-	     if (strcmp(ewin->client.machine, e_machine_name))
+	     if (strcmp(ewin->icccm.wm_machine, e_machine_name))
 		ok = 0;
 	  }
 	if (ok)
@@ -571,9 +590,9 @@ SnapshotEwinBorder(EWin * ewin)
       Efree(sn->border_name);
    sn->border_name = NULL;
    if (ewin->previous_border)
-      sn->border_name = duplicate(ewin->previous_border->name);
-   else if (ewin->border)
-      sn->border_name = duplicate(ewin->border->name);
+      sn->border_name = Estrdup(ewin->previous_border->name);
+   else if (ewin->normal_border)
+      sn->border_name = Estrdup(ewin->normal_border->name);
 }
 
 void
@@ -610,18 +629,17 @@ SnapshotEwinLocation(EWin * ewin)
    if (!sn)
       return;
    sn->use_xy = 1;
-   if (((ewin->pager) || (ewin->ibox)) && (ewin->border))
-     {
-	sn->x = ewin->x + ewin->border->border.left;
-	sn->y = ewin->y + ewin->border->border.top;
-     }
-   else
-     {
-	sn->x = ewin->x;
-	sn->y = ewin->y;
-     }
+   sn->x = ewin->x;
+   sn->y = ewin->y;
    sn->area_x = ewin->area_x;
    sn->area_y = ewin->area_y;
+   if (!ewin->sticky)
+     {
+	sn->x +=
+	   ((desks.desk[ewin->desktop].current_area_x - sn->area_x) * VRoot.w);
+	sn->y +=
+	   ((desks.desk[ewin->desktop].current_area_y - sn->area_y) * VRoot.h);
+     }
 }
 
 void
@@ -685,7 +703,7 @@ SnapshotEwinIcon(EWin * ewin)
    if (sn->iclass_name)
       Efree(sn->iclass_name);
    sn->iclass_name = NULL;
-   /*  sn->iclass_name = duplicate(ewin->border->name); */
+   /*  sn->iclass_name = Estrdup(ewin->border->name); */
 }
 
 void
@@ -710,9 +728,9 @@ SnapshotEwinCmd(EWin * ewin)
    if (!sn)
       return;
 
-   if (ewin->client.machine)
+   if (ewin->icccm.wm_machine)
      {
-	if (strcmp(ewin->client.machine, e_machine_name))
+	if (strcmp(ewin->icccm.wm_machine, e_machine_name))
 	   ok = 0;
      }
    if (ok)
@@ -720,7 +738,7 @@ SnapshotEwinCmd(EWin * ewin)
 	sn->use_cmd = 1;
 	if (sn->cmd)
 	   Efree(sn->cmd);
-	sn->cmd = duplicate(ewin->client.command);
+	sn->cmd = Estrdup(ewin->icccm.wm_command);
      }
 }
 
@@ -746,7 +764,7 @@ SnapshotEwinGroups(EWin * ewin, char onoff)
 	return;
      }
 
-   gwins = ListWinGroupMembersForEwin(ewin, ACTION_NONE, mode.nogroup, &num);
+   gwins = ListWinGroupMembersForEwin(ewin, ACTION_NONE, Mode.nogroup, &num);
    for (i = 0; i < num; i++)
      {
 	if (onoff)
@@ -813,25 +831,22 @@ void
 UnsnapshotEwin(EWin * ewin)
 {
    Snapshot           *sn;
-   char                buf[4096];
 
-   if ((ewin->client.name) && (ewin->client.class))
-      Esnprintf(buf, sizeof(buf), "%s.%s", ewin->client.name,
-		ewin->client.class);
-   else if (ewin->client.title)
-      Esnprintf(buf, sizeof(buf), "TITLE.%s", ewin->client.title);
-   else
-      return;
    if (ewin->snap)
      {
-	ListChangeItemID(LIST_TYPE_SNAPSHOT, ewin->snap, 0);
-	ewin->snap->used = 0;
-	sn = RemoveItem((char *)ewin->snap, 0, LIST_FINDBY_POINTER,
-			LIST_TYPE_SNAPSHOT);
-	ewin->snap = NULL;
+	sn = ewin->snap;
+	UnmatchEwinToSnapInfo(ewin);
+	sn = RemoveItem((char *)sn, 0, LIST_FINDBY_POINTER, LIST_TYPE_SNAPSHOT);
      }
    else
-      sn = RemoveItem(buf, 0, LIST_FINDBY_NAME, LIST_TYPE_SNAPSHOT);
+     {
+	char                buf[4096];
+
+	if (EwinMakeID(ewin, buf, sizeof(buf)))
+	   return;
+	sn = RemoveItem(buf, 0, LIST_FINDBY_BOTH, LIST_TYPE_SNAPSHOT);
+     }
+
    if (sn)
      {
 	if (sn->name)
@@ -856,15 +871,15 @@ SaveSnapInfo(void)
 
 /* save out all snapped info to disk */
 void
-Real_SaveSnapInfo(int dumval, void *dumdat)
+Real_SaveSnapInfo(int dumval __UNUSED__, void *dumdat __UNUSED__)
 {
    Snapshot          **lst, *sn;
    int                 i, j, num;
    char                buf[4096], s[4096];
    FILE               *f;
 
-   if (dumdat)
-      dumval = 0;
+   if (!Mode.wm.save_ok)
+      return;
 
    Etmp(s);
    f = fopen(s, "w");
@@ -887,7 +902,7 @@ Real_SaveSnapInfo(int dumval, void *dumdat)
 	     if (sn->use_desktop)
 		fprintf(f, "DESKTOP: %i\n", sn->desktop);
 	     if (sn->use_xy)
-		fprintf(f, "RES: %i %i\n", root.w, root.h);
+		fprintf(f, "RES: %i %i\n", VRoot.w, VRoot.h);
 	     if (sn->use_wh)
 		fprintf(f, "WH: %i %i\n", sn->w, sn->h);
 	     if (sn->use_xy)
@@ -924,8 +939,11 @@ Real_SaveSnapInfo(int dumval, void *dumdat)
 
    fclose(f);
 
-   Esnprintf(buf, sizeof(buf), "%s.snapshots.%i", GetGenericSMFile(), root.scr);
-   mv(s, buf);
+   Esnprintf(buf, sizeof(buf), "%s.snapshots.%i", GetGenericSMFile(),
+	     VRoot.scr);
+   if (EventDebug(EDBUG_TYPE_SESSION))
+      Eprintf("Real_SaveSnapInfo: %s\n", buf);
+   E_mv(s, buf);
    if (!isfile(buf))
       Alert(_("Error saving snaps file\n"));
 
@@ -961,19 +979,16 @@ LoadSnapInfo(void)
    FILE               *f;
    int                 res_w, res_h;
 
-   Esnprintf(buf, sizeof(buf), "%s.snapshots.%i", GetSMFile(), root.scr);
+   Esnprintf(buf, sizeof(buf), "%s.snapshots.%i", GetSMFile(), VRoot.scr);
    if (!exists(buf))
       Esnprintf(buf, sizeof(buf), "%s.snapshots.%i", GetGenericSMFile(),
-		root.scr);
-#ifndef __EMX__
+		VRoot.scr);
    f = fopen(buf, "r");
-#else
-   f = fopen(buf, "rt");
-#endif
    if (!f)
       return;
-   res_w = root.w;
-   res_h = root.h;
+
+   res_w = VRoot.w;
+   res_h = VRoot.h;
    while (fgets(buf, sizeof(buf), f))
      {
 	/* nuke \n */
@@ -981,22 +996,22 @@ LoadSnapInfo(void)
 	word(buf, 1, s);
 	if (!strcmp(s, "NEW:"))
 	  {
-	     res_w = root.w;
-	     res_h = root.h;
+	     res_w = VRoot.w;
+	     res_h = VRoot.h;
 	     sn = NewSnapshot(atword(buf, 2));
 	  }
 	else if (sn)
 	  {
 	     if (!strcmp(s, "TITLE:"))
-		sn->win_title = duplicate(atword(buf, 2));
+		sn->win_title = Estrdup(atword(buf, 2));
 	     else if (!strcmp(s, "NAME:"))
-		sn->win_name = duplicate(atword(buf, 2));
+		sn->win_name = Estrdup(atword(buf, 2));
 	     else if (!strcmp(s, "CLASS:"))
-		sn->win_class = duplicate(atword(buf, 2));
+		sn->win_class = Estrdup(atword(buf, 2));
 	     else if (!strcmp(s, "CMD:"))
 	       {
 		  sn->use_cmd = 1;
-		  sn->cmd = duplicate(atword(buf, 2));
+		  sn->cmd = Estrdup(atword(buf, 2));
 	       }
 	     else if (!strcmp(s, "DESKTOP:"))
 	       {
@@ -1027,7 +1042,7 @@ LoadSnapInfo(void)
 		  word(buf, 3, s);
 		  sn->y = atoi(s);
 		  /* we changed reses since we last used this snapshot file */
-		  if (res_w != root.w)
+		  if (res_w != VRoot.w)
 		    {
 		       if (sn->use_wh)
 			 {
@@ -1035,15 +1050,15 @@ LoadSnapInfo(void)
 			       sn->x = 0;
 			    else
 			       sn->x =
-				  (sn->x * (root.w - sn->w)) / (res_w - sn->w);
+				  (sn->x * (VRoot.w - sn->w)) / (res_w - sn->w);
 			 }
 		       else
 			 {
-			    if (sn->x >= root.w)
-			       sn->x = root.w - 32;
+			    if (sn->x >= VRoot.w)
+			       sn->x = VRoot.w - 32;
 			 }
 		    }
-		  if (res_h != root.h)
+		  if (res_h != VRoot.h)
 		    {
 		       if (sn->use_wh)
 			 {
@@ -1051,12 +1066,12 @@ LoadSnapInfo(void)
 			       sn->y = 0;
 			    else
 			       sn->y =
-				  (sn->y * (root.h - sn->h)) / (res_h - sn->h);
+				  (sn->y * (VRoot.h - sn->h)) / (res_h - sn->h);
 			 }
 		       else
 			 {
-			    if (sn->y >= root.h)
-			       sn->y = root.h - 32;
+			    if (sn->y >= VRoot.h)
+			       sn->y = VRoot.h - 32;
 			 }
 		    }
 		  word(buf, 4, s);
@@ -1107,9 +1122,9 @@ LoadSnapInfo(void)
 		  sn->shade = atoi(s);
 	       }
 	     else if (!strcmp(s, "BORDER:"))
-		sn->border_name = duplicate(atword(buf, 2));
+		sn->border_name = Estrdup(atword(buf, 2));
 	     else if (!strcmp(s, "ICON:"))
-		sn->iclass_name = duplicate(atword(buf, 2));
+		sn->iclass_name = Estrdup(atword(buf, 2));
 	     else if (!strcmp(s, "GROUP:"))
 	       {
 		  word(buf, 2, s);
@@ -1129,7 +1144,6 @@ void
 MatchEwinToSnapInfo(EWin * ewin)
 {
    Snapshot           *sn;
-   Border             *b;
    int                 i;
 
    sn = FindSnapshot(ewin);
@@ -1139,73 +1153,66 @@ MatchEwinToSnapInfo(EWin * ewin)
    ewin->snap = sn;
    sn->used = 1;
    ListChangeItemID(LIST_TYPE_SNAPSHOT, ewin->snap, 1);
-   if (sn->use_desktop)
-      ewin->desktop = sn->desktop;
+
+   if (sn->use_sticky)
+      ewin->sticky = sn->sticky;
+
+   if (sn->use_desktop && !ewin->sticky)
+      ewin->desktop = DESKTOPS_WRAP_NUM(sn->desktop);
+   else
+      ewin->desktop = desks.current;
+
    if (sn->use_wh)
      {
 	ewin->client.w = sn->w;
 	ewin->client.h = sn->h;
      }
+
    if (sn->use_xy)
      {
 	ewin->client.already_placed = 1;
 	ewin->client.x = sn->x;
 	ewin->client.y = sn->y;
-	if ((!((sn->use_sticky) && (sn->sticky))) && (!ewin->sticky))
+	ewin->client.grav = NorthWestGravity;
+	ewin->area_x = sn->area_x;
+	ewin->area_y = sn->area_y;
+	if (!ewin->sticky)
 	  {
-	     if (sn->use_desktop)
-	       {
-		  ewin->client.x +=
-		     ((sn->area_x - desks.desk[ewin->desktop].current_area_x) *
-		      root.w);
-		  ewin->client.y +=
-		     ((sn->area_y - desks.desk[ewin->desktop].current_area_y) *
-		      root.h);
-	       }
-	     else
-	       {
-		  ewin->client.x +=
-		     ((sn->area_x - desks.desk[desks.current].current_area_x) *
-		      root.w);
-		  ewin->client.y +=
-		     ((sn->area_y - desks.desk[desks.current].current_area_y) *
-		      root.h);
-	       }
+	     ewin->client.x +=
+		((sn->area_x - desks.desk[ewin->desktop].current_area_x) *
+		 VRoot.w);
+	     ewin->client.y +=
+		((sn->area_y - desks.desk[ewin->desktop].current_area_y) *
+		 VRoot.h);
 	  }
-	ewin->x = ewin->client.x;
-	ewin->y = ewin->client.y;
 	EMoveResizeWindow(disp, ewin->client.win, ewin->client.x,
 			  ewin->client.y, ewin->client.w, ewin->client.h);
      }
+
    if (sn->use_layer)
       ewin->layer = sn->layer;
-   if (sn->use_sticky)
-      ewin->sticky = sn->sticky;
+
    if (sn->use_skiplists)
      {
 	ewin->skipfocus = sn->skipfocus;
 	ewin->skiptask = sn->skiptask;
 	ewin->skipwinlist = sn->skipwinlist;
      }
+
    if (sn->use_neverfocus)
       ewin->neverfocus = sn->neverfocus;
+
    if (sn->use_shade)
       ewin->shaded = sn->shade;
+
    if (sn->iclass_name)
      {
 	/* FIXME: fill this in */
      }
+
    if (sn->border_name)
-     {
-	b = (Border *) FindItem(sn->border_name, 0, LIST_FINDBY_NAME,
-				LIST_TYPE_BORDER);
-	if (b)
-	  {
-	     ewin->border_new = 1;
-	     ewin->border = b;
-	     SetFrameProperty(ewin);
-	  }
-     }
+      EwinSetBorderByName(ewin, sn->border_name, 0);
+
    if (sn->groups)
      {
 	for (i = 0; i < sn->num_groups; i++)
@@ -1226,179 +1233,26 @@ MatchEwinToSnapInfo(EWin * ewin)
 		AddEwinToGroup(ewin, g);
 	  }
      }
+
+   if (EventDebug(EDBUG_TYPE_SNAPS))
+      Eprintf("Snap get snap  %#lx: %4d+%4d %4dx%4d: %s\n",
+	      ewin->client.win, ewin->client.x, ewin->client.y,
+	      ewin->client.w, ewin->client.h, EwinGetTitle(ewin));
 }
 
+/* make a client window conform to snapshot info */
 void
-MatchToSnapInfoPager(Pager * p)
-{
-   XClassHint          hint;
-   char                buf[1024];
-   Snapshot           *sn = NULL;
-
-   if ((!XGetClassHint(disp, p->win, &hint)))
-      return;
-   if ((hint.res_name) && (hint.res_class))
-     {
-	Esnprintf(buf, sizeof(buf), "%s.%s", hint.res_name, hint.res_class);
-	sn = FindItem(buf, 0, LIST_FINDBY_BOTH, LIST_TYPE_SNAPSHOT);
-     }
-   if (hint.res_name)
-      XFree(hint.res_name);
-   if (hint.res_class)
-      XFree(hint.res_class);
-   if (!sn)
-      return;
-   if (sn->use_xy)
-      EMoveWindow(disp, p->win, sn->x, sn->y);
-   if (sn->use_wh)
-      PagerResize(p, sn->w, sn->h);
-}
-
-void
-MatchToSnapInfoIconbox(Iconbox * ib)
-{
-   XClassHint          hint;
-   char                buf[1024];
-   Snapshot           *sn = NULL;
-
-   if ((!XGetClassHint(disp, ib->win, &hint)))
-      return;
-   if ((hint.res_name) && (hint.res_class))
-     {
-	Esnprintf(buf, sizeof(buf), "%s.%s", hint.res_name, hint.res_class);
-	sn = FindItem(buf, 0, LIST_FINDBY_BOTH, LIST_TYPE_SNAPSHOT);
-     }
-   if (hint.res_name)
-      XFree(hint.res_name);
-   if (hint.res_class)
-      XFree(hint.res_class);
-   if (!sn)
-      return;
-   if (sn->use_xy)
-      EMoveWindow(disp, ib->win, sn->x, sn->y);
-   if (sn->use_wh)
-      IconboxResize(ib, sn->w, sn->h);
-}
-
-void
-MatchEwinToSnapInfoAfter(EWin * ewin)
+UnmatchEwinToSnapInfo(EWin * ewin)
 {
    Snapshot           *sn;
-   Border             *b;
-   int                 i;
 
-   sn = FindSnapshot(ewin);
-   if (!sn)
+   sn = ewin->snap;
+   if (sn == NULL)
       return;
 
-   ewin->snap = sn;
-   sn->used = 1;
-   ListChangeItemID(LIST_TYPE_SNAPSHOT, ewin->snap, 1);
-   if (sn->use_desktop)
-      MoveEwinToDesktop(ewin, sn->desktop);
-   if (sn->use_wh)
-      ResizeEwin(ewin, sn->w, sn->h);
-   if (sn->use_xy)
-     {
-	if ((ewin->pager) || (ewin->ibox))
-	   MoveEwin(ewin, sn->x - ewin->border->border.left,
-		    sn->y - ewin->border->border.top);
-	else
-	  {
-	     if ((!((sn->use_sticky) && (sn->sticky))) && (!ewin->sticky))
-	       {
-		  if (sn->use_desktop)
-		     MoveEwin(ewin,
-			      sn->x +
-			      ((sn->area_x -
-				desks.desk[ewin->desktop].current_area_x) *
-			       root.w),
-			      sn->y +
-			      ((sn->area_y
-				-
-				desks.desk[ewin->desktop].current_area_y) *
-			       root.h));
-		  else
-		     MoveEwin(ewin,
-			      sn->x +
-			      ((sn->area_x
-				-
-				desks.desk[desks.current].current_area_x) *
-			       root.w),
-			      sn->y +
-			      ((sn->area_y
-				-
-				desks.desk[desks.current].current_area_y) *
-			       root.h));
-	       }
-	     else
-		MoveEwin(ewin, sn->x, sn->y);
-	  }
-     }
-   if (sn->use_layer)
-     {
-	ewin->layer = sn->layer;
-	RaiseEwin(ewin);
-     }
-   if (sn->use_sticky)
-     {
-	if (sn->sticky)
-	   MakeWindowSticky(ewin);
-	else
-	   MakeWindowUnSticky(ewin);
-     }
-   if (sn->use_skiplists)
-     {
-	ewin->skipfocus = sn->skipfocus;
-	ewin->skiptask = sn->skiptask;
-	ewin->skipwinlist = sn->skipwinlist;
-     }
-   if (sn->use_neverfocus)
-      ewin->neverfocus = sn->neverfocus;
-   if (sn->use_shade)
-     {
-	if (sn->shade)
-	   InstantShadeEwin(ewin);
-	else
-	   InstantUnShadeEwin(ewin);
-     }
-   if (sn->iclass_name)
-     {
-	/* FIXME: fill this in */
-     }
-   if (sn->border_name)
-     {
-	b = (Border *) FindItem(sn->border_name, 0, LIST_FINDBY_NAME,
-				LIST_TYPE_BORDER);
-	if (b)
-	  {
-	     ewin->border_new = 1;
-	     SetEwinToBorder(ewin, b);
-	     ICCCM_MatchSize(ewin);
-	     MoveResizeEwin(ewin, ewin->x, ewin->y, ewin->client.w,
-			    ewin->client.h);
-	  }
-     }
-   if (sn->groups)
-     {
-	for (i = 0; i < sn->num_groups; i++)
-	  {
-	     Group              *g;
-
-	     g = (Group *) FindItem(NULL, sn->groups[i], LIST_FINDBY_ID,
-				    LIST_TYPE_GROUP);
-	     if (!g)
-	       {
-		  BuildWindowGroup(&ewin, 1);
-		  ewin->groups[ewin->num_groups - 1]->index = sn->groups[i];
-		  ListChangeItemID(LIST_TYPE_GROUP,
-				   ewin->groups[ewin->num_groups - 1],
-				   sn->groups[i]);
-	       }
-	     else
-		AddEwinToGroup(ewin, g);
-	  }
-     }
+   ewin->snap = NULL;
+   sn->used = 0;
+   ListChangeItemID(LIST_TYPE_SNAPSHOT, sn, 0);
 }
 
 void
@@ -1406,6 +1260,10 @@ RememberImportantInfoForEwin(EWin * ewin)
 {
    if ((ewin->pager) || (ewin->ibox))
      {
+	if (EventDebug(EDBUG_TYPE_SNAPS))
+	   Eprintf("RememberImportantInfoForEwin  %#lx %s\n",
+		   ewin->client.win, EwinGetTitle(ewin));
+
 	SnapshotEwinBorder(ewin);
 	SnapshotEwinDesktop(ewin);
 	SnapshotEwinSize(ewin);
@@ -1419,33 +1277,3 @@ RememberImportantInfoForEwin(EWin * ewin)
 	SaveSnapInfo();
      }
 }
-
-void
-RememberImportantInfoForEwins(EWin * ewin)
-{
-   int                 i, num;
-   EWin              **gwins;
-
-   gwins = ListWinGroupMembersForEwin(ewin, ACTION_MOVE, mode.nogroup, &num);
-   if (gwins)
-     {
-	for (i = 0; i < num; i++)
-	  {
-	     if ((gwins[i]->pager) || (gwins[i]->ibox))
-	       {
-		  SnapshotEwinBorder(gwins[i]);
-		  SnapshotEwinDesktop(gwins[i]);
-		  SnapshotEwinSize(gwins[i]);
-		  SnapshotEwinLocation(gwins[i]);
-		  SnapshotEwinLayer(gwins[i]);
-		  SnapshotEwinSticky(gwins[i]);
-		  SnapshotEwinShade(gwins[i]);
-		  SnapshotEwinGroups(gwins[i], gwins[i]->num_groups);
-		  SnapshotEwinSkipLists(gwins[i]);
-		  SnapshotEwinNeverFocus(gwins[i]);
-		  SaveSnapInfo();
-	       }
-	  }
-	Efree(gwins);
-     }
-}
