@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,9 +21,10 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include <sys/time.h>
+#include <time.h>
 
-#ifndef __EMX__
-int
+static int
 AddEToFile(char *file)
 {
    FILE               *f1, *f2;
@@ -31,7 +32,7 @@ AddEToFile(char *file)
    char               *s1, *s2;
    char                hase = 0;
    char                foundwm = 0;
-   char               *wms[] = {
+   const char         *wms[] = {
       "wmaker", "afterstep", "fvwm", "fvwm2", "twm", "mwm", "vtwm", "ctwm",
       "gwm",
       "mlvwm", "kwm", "olwm", "wm2", "wmx", "olvwm", "9wm", "blackbox", "awm",
@@ -49,7 +50,7 @@ AddEToFile(char *file)
 
    Esnprintf(tmp, 2048, "%s/estrt_%i",
 	     (getenv("TMPDIR") == NULL) ? "/tmp" : getenv("TMPDIR"),
-	     time(NULL));
+	     (unsigned)time(NULL));
    f1 = fopen(file, "r");
    if (!f1)
       EDBUG_RETURN(0);
@@ -79,8 +80,7 @@ AddEToFile(char *file)
 		       fprintf(f2, "#%s", s);
 		       fprintf(f2,
 			       "\n# Enlightenment inserted Execution string here\n");
-		       fprintf(f2, "exec %s/enlightenment\n\n",
-			       ENLIGHTENMENT_BIN);
+		       fprintf(f2, "exec %s/enlightenment\n\n", EDirBin());
 		       foundwm = 1;
 		       i = wmnum + 1;
 		    }
@@ -103,19 +103,19 @@ AddEToFile(char *file)
 	     EDBUG_RETURN(0);
 	  }
 	fprintf(f2, "\n# Enlightenment inserted Execution string here\n");
-	fprintf(f2, "exec %s/enlightenment\n\n", ENLIGHTENMENT_BIN);
+	fprintf(f2, "exec %s/enlightenment\n\n", EDirBin());
 	while (fgets(s, 2048, f1))
 	   fprintf(f2, "%s", s);
 	fclose(f1);
 	fclose(f2);
      }
    if (!hase)
-      cp(tmp, file);
-   rm(tmp);
+      E_cp(tmp, file);
+   E_rm(tmp);
    EDBUG_RETURN(1);
 }
 
-int
+static int
 CreateEFile(char *file)
 {
    FILE               *f;
@@ -125,16 +125,14 @@ CreateEFile(char *file)
    if (!f)
       EDBUG_RETURN(0);
    fprintf(f, "# Enlightenment inserted Execution string here\n");
-   fprintf(f, "exec %s/enlightenment\n", ENLIGHTENMENT_BIN);
+   fprintf(f, "exec %s/enlightenment\n", EDirBin());
    fclose(f);
    EDBUG_RETURN(1);
 }
-#endif
 
 void
 AddE()
 {
-#ifndef __EMX__
    char               *h;
    char                s[1024];
    int                 val;
@@ -160,7 +158,6 @@ AddE()
    if (h)
       Efree(h);
    EDBUG_RETURN_;
-#endif
 }
 
 void
@@ -192,38 +189,44 @@ CreateStartupDisplay(char start)
 
 	if (bg_sideways)
 	  {
-	     w1 = ECreateWindow(root.win, (root.w / 2), 0, root.w, root.h, 1);
-	     w2 = ECreateWindow(root.win, -(root.w / 2), 0, root.w, root.h, 1);
-	     win1 = ECreateWindow(w1, -(root.w / 2), 0, root.w, root.h, 0);
-	     win2 = ECreateWindow(w2, (root.w / 2), 0, root.w, root.h, 0);
+	     w1 =
+		ECreateWindow(VRoot.win, (VRoot.w / 2), 0, VRoot.w, VRoot.h, 1);
+	     w2 =
+		ECreateWindow(VRoot.win, -(VRoot.w / 2), 0, VRoot.w, VRoot.h,
+			      1);
+	     win1 = ECreateWindow(w1, -(VRoot.w / 2), 0, VRoot.w, VRoot.h, 0);
+	     win2 = ECreateWindow(w2, (VRoot.w / 2), 0, VRoot.w, VRoot.h, 0);
 	  }
 	else
 	  {
-	     w1 = ECreateWindow(root.win, 0, -(root.h / 2), root.w, root.h, 1);
-	     w2 = ECreateWindow(root.win, 0, (root.h / 2), root.w, root.h, 1);
-	     win1 = ECreateWindow(w1, 0, (root.h / 2), root.w, root.h, 0);
-	     win2 = ECreateWindow(w2, 0, -(root.h / 2), root.w, root.h, 0);
+	     w1 =
+		ECreateWindow(VRoot.win, 0, -(VRoot.h / 2), VRoot.w, VRoot.h,
+			      1);
+	     w2 =
+		ECreateWindow(VRoot.win, 0, (VRoot.h / 2), VRoot.w, VRoot.h, 1);
+	     win1 = ECreateWindow(w1, 0, (VRoot.h / 2), VRoot.w, VRoot.h, 0);
+	     win2 = ECreateWindow(w2, 0, -(VRoot.h / 2), VRoot.w, VRoot.h, 0);
 	  }
 
 	EMapWindow(disp, win1);
 	EMapWindow(disp, win2);
 
-	b1 = ECreateWindow(w1, 0, root.h - desks.dragbar_width, root.w,
-			   desks.dragbar_width, 0);
-	b2 = ECreateWindow(w2, 0, 0, root.w, desks.dragbar_width, 0);
+	b1 = ECreateWindow(w1, 0, VRoot.h - Conf.desks.dragbar_width, VRoot.w,
+			   Conf.desks.dragbar_width, 0);
+	b2 = ECreateWindow(w2, 0, 0, VRoot.w, Conf.desks.dragbar_width, 0);
 	EMapRaised(disp, b1);
 	EMapRaised(disp, b2);
 
-	pq = queue_up;
-	queue_up = 0;
-	IclassApply(ic, b1, root.w, desks.dragbar_width, 0, 0, 0, 0);
-	IclassApply(ic, b2, root.w, desks.dragbar_width, 0, 0, 0, 0);
-	queue_up = pq;
-	SetBackgroundTo(pImlibData, win1, bg, 1);
-	SetBackgroundTo(pImlibData, win2, bg, 1);
-	if (bg->pmap)
-	   Imlib_free_pixmap(pImlibData, bg->pmap);
-	bg->pmap = 0;
+	pq = Mode.queue_up;
+	Mode.queue_up = 0;
+	IclassApply(ic, b1, VRoot.w, Conf.desks.dragbar_width, 0, 0, 0, 0,
+		    ST_UNKNWN);
+	IclassApply(ic, b2, VRoot.w, Conf.desks.dragbar_width, 0, 0, 0, 0,
+		    ST_UNKNWN);
+	Mode.queue_up = pq;
+	BackgroundApply(bg, win1, 1);
+	BackgroundApply(bg, win2, 1);
+	BackgroundImagesFree(bg, 1);
 	init_win1 = w1;
 	init_win2 = w2;
 	EMapRaised(disp, w1);
@@ -242,7 +245,7 @@ CreateStartupDisplay(char start)
 	if ((!ic) || (!bg))
 	   EDBUG_RETURN_;
 
-	speed = mode.slidespeedcleanup / 2;
+	speed = Conf.slidespeedcleanup / 2;
 	spd = 16;
 	min = 2;
 	fy = 0;
@@ -251,7 +254,7 @@ CreateStartupDisplay(char start)
 	  {
 	     if (bg_sideways)
 	       {		/* so we can have two different slide methods */
-		  ty = (root.w / 2);
+		  ty = (VRoot.w / 2);
 		  xOffset = ((fy * (1024 - k)) + (ty * k)) >> 10;
 		  x = ty;
 		  yOffset = 0;
@@ -259,7 +262,7 @@ CreateStartupDisplay(char start)
 	       }
 	     else
 	       {
-		  ty = (root.h / 2);
+		  ty = (VRoot.h / 2);
 		  xOffset = 0;
 		  x = 0;
 		  yOffset = ((fy * (1024 - k)) + (ty * k)) >> 10;
