@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -40,7 +40,7 @@ static char         zoom_can = 0;
 static void
 FillStdVidModes(void)
 {
-   XF86VidModeGetAllModeLines(disp, root.scr, &std_vid_modes_num,
+   XF86VidModeGetAllModeLines(disp, VRoot.scr, &std_vid_modes_num,
 			      &std_vid_modes);
 }
 
@@ -52,16 +52,15 @@ FindMode(int w, int h)
 
    for (i = 0; i < std_vid_modes_num; i++)
      {
-	int                 close = 0x7fffffff;
+	int                 value = 0x7fffffff;
 
-	if ((std_vid_modes[i]->hdisplay >= w)
-	    && (std_vid_modes[i]->vdisplay >= h))
-	   close =
-	      ((std_vid_modes[i]->hdisplay - w) +
-	       (std_vid_modes[i]->vdisplay - h));
-	if (close < closest)
+	if ((std_vid_modes[i]->hdisplay >= w) &&
+	    (std_vid_modes[i]->vdisplay >= h))
+	   value = ((std_vid_modes[i]->hdisplay - w) +
+		    (std_vid_modes[i]->vdisplay - h));
+	if (value < closest)
 	  {
-	     closest = close;
+	     closest = value;
 	     chosen = std_vid_modes[i];
 	  }
      }
@@ -69,7 +68,7 @@ FindMode(int w, int h)
 }
 
 static int
-GetModeIndex(int dotclock, XF86VidModeModeLine * line)
+GetModeIndex(unsigned int dotclock, XF86VidModeModeLine * line)
 {
    int                 i;
    const XF86VidModeModeInfo *info;
@@ -96,28 +95,28 @@ SwitchRes(char inout, int x, int y, int w, int h)
 	XF86VidModeModeLine curmode;
 	int                 dotclock;
 
-	if (!XF86VidModeGetModeLine(disp, root.scr, &dotclock, &curmode))
+	if (!XF86VidModeGetModeLine(disp, VRoot.scr, &dotclock, &curmode))
 	   return mode;
-	XF86VidModeGetViewPort(disp, root.scr, &vp_x, &vp_y);
+	XF86VidModeGetViewPort(disp, VRoot.scr, &vp_x, &vp_y);
 
 	mode = FindMode(w, h);
 	if (mode)
 	  {
-	     XF86VidModeLockModeSwitch(disp, root.scr, 0);
+	     XF86VidModeLockModeSwitch(disp, VRoot.scr, 0);
 	     std_vid_mode_cur = GetModeIndex(dotclock, &curmode);
-	     XF86VidModeSwitchToMode(disp, root.scr, mode);
-	     XF86VidModeSetViewPort(disp, root.scr, x, y);
-	     XF86VidModeLockModeSwitch(disp, root.scr, 1);
+	     XF86VidModeSwitchToMode(disp, VRoot.scr, mode);
+	     XF86VidModeSetViewPort(disp, VRoot.scr, x, y);
+	     XF86VidModeLockModeSwitch(disp, VRoot.scr, 1);
 	  }
      }
    else
      {
 	mode = std_vid_modes[std_vid_mode_cur];
-	XF86VidModeLockModeSwitch(disp, root.scr, 0);
-	XF86VidModeSwitchToMode(disp, root.scr, mode);
-	XF86VidModeSetViewPort(disp, root.scr, vp_x, vp_y);
+	XF86VidModeLockModeSwitch(disp, VRoot.scr, 0);
+	XF86VidModeSwitchToMode(disp, VRoot.scr, mode);
+	XF86VidModeSetViewPort(disp, VRoot.scr, vp_x, vp_y);
 #if 0				/* No, don't lock or we can't switch resolution */
-	XF86VidModeLockModeSwitch(disp, root.scr, 1);
+	XF86VidModeLockModeSwitch(disp, VRoot.scr, 1);
 #endif
      }
    return mode;
@@ -183,9 +182,9 @@ ZoomMask(int x, int y, int w, int h)
    if (x < 0 || y < 0 || w <= 0 || h <= 0)
       return 0;
 
-   win = ECreateWindow(root.win, x, y, w, h, 0);
+   win = ECreateWindow(VRoot.win, x, y, w, h, 0);
    XSetWindowBackgroundPixmap(disp, win, None);
-   XSetWindowBackground(disp, win, BlackPixel(disp, root.scr));
+   XSetWindowBackground(disp, win, BlackPixel(disp, VRoot.scr));
    XRaiseWindow(disp, win);
    XMapWindow(disp, win);
 
@@ -209,13 +208,13 @@ Zoom(EWin * ewin)
 	     MoveEwin(ewin, zoom_last_x, zoom_last_y);
 	     ICCCM_Configure(ewin);
 	     if (zoom_mask_1)
-		XDestroyWindow(disp, zoom_mask_1);
+		EDestroyWindow(disp, zoom_mask_1);
 	     if (zoom_mask_2)
-		XDestroyWindow(disp, zoom_mask_2);
+		EDestroyWindow(disp, zoom_mask_2);
 	     if (zoom_mask_3)
-		XDestroyWindow(disp, zoom_mask_3);
+		EDestroyWindow(disp, zoom_mask_3);
 	     if (zoom_mask_4)
-		XDestroyWindow(disp, zoom_mask_4);
+		EDestroyWindow(disp, zoom_mask_4);
 	     SwitchRes(0, 0, 0, 0, 0);
 	     XWarpPointer(disp, None, ewin->client.win, 0, 0, 0, 0,
 			  ewin->client.w / 2, ewin->client.h / 2);
@@ -249,10 +248,10 @@ Zoom(EWin * ewin)
 	MoveEwin(ewin, -ewin->border->border.left + x1,
 		 -ewin->border->border.top + y1);
 	ICCCM_Configure(ewin);
-	FocusToEWin(ewin);
+	FocusToEWin(ewin, FOCUS_SET);
 	XWarpPointer(disp, None, ewin->client.win, 0, 0, 0, 0,
 		     ewin->client.w / 2, ewin->client.h / 2);
-#if 0
+#if 0				/* Doesn't work as intended */
 	XGrabPointer(disp, ewin->client.win, True,
 		     ButtonPressMask | ButtonReleaseMask |
 		     PointerMotionMask | ButtonMotionMask |
